"""Library defining the interface to the generative firewall."""
import atexit
from pathlib import Path
from typing import List, Optional, Union

from urllib3.util import Retry

from rime_sdk.client import RETRY_HTTP_STATUS
from rime_sdk.internal.file_upload import GenerativeFirewallFileUploader
from rime_sdk.internal.rest_error_handler import RESTErrorHandler
from rime_sdk.swagger.swagger_client import (
    ApiClient,
    Configuration,
    FirewallApi,
    FirewallConfigurationApi,
    GenerativefirewallFirewallConfig,
    GenerativefirewallValidateRequest,
    ValidateRequestInput,
    ValidateRequestOutput,
)

_DEFAULT_CHANNEL_TIMEOUT = 60.0


class GenerativeFirewall:
    """An interface to a Generative Firewall object.

    To initialize the GenerativeFirewall, provide the address of your RIME instance.

    Args:
        domain: str
            The base domain/address of the RIME service.
        api_key: str
            The API key used to authenticate to RIME services.
        channel_timeout: float
            The amount of time in seconds to wait for responses from the cluster.

    Example:
        .. code-block:: python

            firewall = GenerativeFirewall("my_vpc.rime.com", "api-key")
    """

    def __init__(
        self,
        domain: str,
        api_key: str = "",
        channel_timeout: float = _DEFAULT_CHANNEL_TIMEOUT,
    ):
        """Create a new Client connected to the services available at `domain`."""
        configuration = Configuration()
        configuration.api_key["X-Firewall-Api-Key"] = api_key
        if domain.endswith("/"):
            domain = domain[:-1]
        if not domain.startswith("https://") and not domain.startswith("http://"):
            domain = "https://" + domain
        configuration.host = domain
        self._api_client = ApiClient(configuration)
        # Prevent race condition in pool.close() triggered by swagger generated code
        atexit.register(self._api_client.pool.close)
        # Sets the timeout and hardcoded retries parameter for the api client.
        self._api_client.rest_client.pool_manager.connection_pool_kw[
            "timeout"
        ] = channel_timeout
        self._api_client.rest_client.pool_manager.connection_pool_kw["retries"] = Retry(
            total=3, status_forcelist=RETRY_HTTP_STATUS
        )
        self._firewall_client = FirewallApi(self._api_client)
        self._configuration_client = FirewallConfigurationApi(self._api_client)

    def validate(
        self, input: Optional[str] = None, output: Optional[str] = None
    ) -> List[dict]:
        """Validate model input and/or output text.

        Args:
            input: Optional[str]
                The user input text to validate.
            output: Optional[str]
                The model output text to validate.

        Returns:
            List[dict]:
                A list of validation results, each of which is a dictionary of rule
                results for the input or output text.

        Raises:
            ValueError:
                If neither input nor output text is provided or when there was an error
                in the validation request to the RIME backend.

        Example:
            .. code-block:: python

                results = firewall.validate(
                    input="Hello!", output="Hi, how can I help you?"
                )
        """
        if input is None and output is None:
            raise ValueError("Must provide either input or output text to validate.")

        body = GenerativefirewallValidateRequest(
            input=ValidateRequestInput(user_input_text=input),
            output=ValidateRequestOutput(output_text=output),
        )
        with RESTErrorHandler():
            response = self._firewall_client.firewall_validate(body=body)
        return [res.to_dict() for res in response.results]

    def upload_file(self, file_path: Union[Path, str]) -> str:
        """Upload a file to make it accessible to the RIME cluster.

        The uploaded file is stored in the RIME cluster in a blob store
        using its file name.

        Args:
            file_path: Union[Path, str]
                Path to the file to be uploaded to RIME's blob store.

        Returns:
            str:
                A reference to the uploaded file's location in the blob store. This
                reference can be used to refer to that object when writing RIME configs.
                Please store this reference for future access to the file.

        Raises:
            FileNotFoundError
                When the path ``file_path`` does not exist.
            IOError
                When ``file_path`` is not a file.
            ValueError
                When there was an error in obtaining a blobstore location from the
                RIME backend or in uploading ``file_path`` to RIME's blob store.
                When the file upload fails, the incomplete file is
                NOT automatically deleted.

        Example:
             .. code-block:: python

                uploaded_file_path = firewall.upload_file(file_path)
        """
        if isinstance(file_path, str):
            file_path = Path(file_path)

        with RESTErrorHandler():
            file_uploader = GenerativeFirewallFileUploader(self._api_client)
            return file_uploader.upload_file(file_path)

    def update_config(self, firewall_config: dict) -> dict:
        """Update the firewall configuration.

        The update is performed via a PATCH request by merging the provided
        `firewall_config` with the existing config, meaning that any fields not
        explicitly provided in `firewall_config` will not be overwritten.

        Args:
            firewall_config: dict
                A dictionary containing the firewall configuration components to update.

        Returns:
            dict:
                The updated firewall configuration.

        Raises:
            ValueError:
                If `firewall_config` is empty or contains unexpected keys, or when there
                was an error in the update request to the RIME backend.

        Example:
             .. code-block:: python

                updated_config = firewall.update_config(firewall_config)
        """
        _config = firewall_config.copy()
        body = GenerativefirewallFirewallConfig(
            fact_sheet_url=_config.pop("fact_sheet_url", None),
            sensitive_terms_url=_config.pop("sensitive_terms_url", None),
            topic=_config.pop("topic", None),
        )

        if _config:
            raise ValueError(
                "Found unexpected keys in firewall_config: " f"{list(_config.keys())}"
            )
        if all(val is None for val in body.to_dict().values()):
            raise ValueError("Must provide a non-empty firewall_config to update.")

        with RESTErrorHandler():
            response = (
                self._configuration_client.firewall_configuration_configure_firewall(
                    body
                )
            )
        return response.config.to_dict()



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class NotificationNotification(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'RimeUUID',
        'object_type': 'NotificationObjectType',
        'object_id': 'str',
        'notification_type': 'NotificationNotificationType',
        'config': 'SchemanotificationConfig',
        'webhooks': 'list[NotificationWebhookConfig]',
        'emails': 'list[str]',
        'last_send_time': 'datetime'
    }

    attribute_map = {
        'id': 'id',
        'object_type': 'objectType',
        'object_id': 'objectId',
        'notification_type': 'notificationType',
        'config': 'config',
        'webhooks': 'webhooks',
        'emails': 'emails',
        'last_send_time': 'lastSendTime'
    }

    def __init__(self, id=None, object_type=None, object_id=None, notification_type=None, config=None, webhooks=None, emails=None, last_send_time=None):  # noqa: E501
        """NotificationNotification - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._object_type = None
        self._object_id = None
        self._notification_type = None
        self._config = None
        self._webhooks = None
        self._emails = None
        self._last_send_time = None
        self.discriminator = None
        if id is not None:
            self.id = id
        if object_type is not None:
            self.object_type = object_type
        if object_id is not None:
            self.object_id = object_id
        if notification_type is not None:
            self.notification_type = notification_type
        if config is not None:
            self.config = config
        if webhooks is not None:
            self.webhooks = webhooks
        if emails is not None:
            self.emails = emails
        if last_send_time is not None:
            self.last_send_time = last_send_time

    @property
    def id(self):
        """Gets the id of this NotificationNotification.  # noqa: E501


        :return: The id of this NotificationNotification.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this NotificationNotification.


        :param id: The id of this NotificationNotification.  # noqa: E501
        :type: RimeUUID
        """

        self._id = id

    @property
    def object_type(self):
        """Gets the object_type of this NotificationNotification.  # noqa: E501


        :return: The object_type of this NotificationNotification.  # noqa: E501
        :rtype: NotificationObjectType
        """
        return self._object_type

    @object_type.setter
    def object_type(self, object_type):
        """Sets the object_type of this NotificationNotification.


        :param object_type: The object_type of this NotificationNotification.  # noqa: E501
        :type: NotificationObjectType
        """

        self._object_type = object_type

    @property
    def object_id(self):
        """Gets the object_id of this NotificationNotification.  # noqa: E501

        Uniquely specifies the object for the notification.  # noqa: E501

        :return: The object_id of this NotificationNotification.  # noqa: E501
        :rtype: str
        """
        return self._object_id

    @object_id.setter
    def object_id(self, object_id):
        """Sets the object_id of this NotificationNotification.

        Uniquely specifies the object for the notification.  # noqa: E501

        :param object_id: The object_id of this NotificationNotification.  # noqa: E501
        :type: str
        """

        self._object_id = object_id

    @property
    def notification_type(self):
        """Gets the notification_type of this NotificationNotification.  # noqa: E501


        :return: The notification_type of this NotificationNotification.  # noqa: E501
        :rtype: NotificationNotificationType
        """
        return self._notification_type

    @notification_type.setter
    def notification_type(self, notification_type):
        """Sets the notification_type of this NotificationNotification.


        :param notification_type: The notification_type of this NotificationNotification.  # noqa: E501
        :type: NotificationNotificationType
        """

        self._notification_type = notification_type

    @property
    def config(self):
        """Gets the config of this NotificationNotification.  # noqa: E501


        :return: The config of this NotificationNotification.  # noqa: E501
        :rtype: SchemanotificationConfig
        """
        return self._config

    @config.setter
    def config(self, config):
        """Sets the config of this NotificationNotification.


        :param config: The config of this NotificationNotification.  # noqa: E501
        :type: SchemanotificationConfig
        """

        self._config = config

    @property
    def webhooks(self):
        """Gets the webhooks of this NotificationNotification.  # noqa: E501

        Webhooks are a list of destination webhooks to which notifications will be sent.  # noqa: E501

        :return: The webhooks of this NotificationNotification.  # noqa: E501
        :rtype: list[NotificationWebhookConfig]
        """
        return self._webhooks

    @webhooks.setter
    def webhooks(self, webhooks):
        """Sets the webhooks of this NotificationNotification.

        Webhooks are a list of destination webhooks to which notifications will be sent.  # noqa: E501

        :param webhooks: The webhooks of this NotificationNotification.  # noqa: E501
        :type: list[NotificationWebhookConfig]
        """

        self._webhooks = webhooks

    @property
    def emails(self):
        """Gets the emails of this NotificationNotification.  # noqa: E501

        Emails specify the list of emails to which notifications will be sent.  # noqa: E501

        :return: The emails of this NotificationNotification.  # noqa: E501
        :rtype: list[str]
        """
        return self._emails

    @emails.setter
    def emails(self, emails):
        """Sets the emails of this NotificationNotification.

        Emails specify the list of emails to which notifications will be sent.  # noqa: E501

        :param emails: The emails of this NotificationNotification.  # noqa: E501
        :type: list[str]
        """

        self._emails = emails

    @property
    def last_send_time(self):
        """Gets the last_send_time of this NotificationNotification.  # noqa: E501

        Last send time is the last time the notification was successfully sent.  # noqa: E501

        :return: The last_send_time of this NotificationNotification.  # noqa: E501
        :rtype: datetime
        """
        return self._last_send_time

    @last_send_time.setter
    def last_send_time(self, last_send_time):
        """Sets the last_send_time of this NotificationNotification.

        Last send time is the last time the notification was successfully sent.  # noqa: E501

        :param last_send_time: The last_send_time of this NotificationNotification.  # noqa: E501
        :type: datetime
        """

        self._last_send_time = last_send_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NotificationNotification, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NotificationNotification):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GenerativefirewallFirewallConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'fact_sheet_url': 'str',
        'sensitive_terms_url': 'str',
        'topic': 'str'
    }

    attribute_map = {
        'fact_sheet_url': 'factSheetUrl',
        'sensitive_terms_url': 'sensitiveTermsUrl',
        'topic': 'topic'
    }

    def __init__(self, fact_sheet_url=None, sensitive_terms_url=None, topic=None):  # noqa: E501
        """GenerativefirewallFirewallConfig - a model defined in Swagger"""  # noqa: E501
        self._fact_sheet_url = None
        self._sensitive_terms_url = None
        self._topic = None
        self.discriminator = None
        if fact_sheet_url is not None:
            self.fact_sheet_url = fact_sheet_url
        if sensitive_terms_url is not None:
            self.sensitive_terms_url = sensitive_terms_url
        if topic is not None:
            self.topic = topic

    @property
    def fact_sheet_url(self):
        """Gets the fact_sheet_url of this GenerativefirewallFirewallConfig.  # noqa: E501

        Fact sheet URL describes the location of a fact sheet to use in validation rules around correctness. If not provided, the factual inconsistency rules will not run.  # noqa: E501

        :return: The fact_sheet_url of this GenerativefirewallFirewallConfig.  # noqa: E501
        :rtype: str
        """
        return self._fact_sheet_url

    @fact_sheet_url.setter
    def fact_sheet_url(self, fact_sheet_url):
        """Sets the fact_sheet_url of this GenerativefirewallFirewallConfig.

        Fact sheet URL describes the location of a fact sheet to use in validation rules around correctness. If not provided, the factual inconsistency rules will not run.  # noqa: E501

        :param fact_sheet_url: The fact_sheet_url of this GenerativefirewallFirewallConfig.  # noqa: E501
        :type: str
        """

        self._fact_sheet_url = fact_sheet_url

    @property
    def sensitive_terms_url(self):
        """Gets the sensitive_terms_url of this GenerativefirewallFirewallConfig.  # noqa: E501

        Sensitive terms URL describes the location of a sensitive terms sheet to use in validation rules around PII. If not provided, the PII rules will not run.  # noqa: E501

        :return: The sensitive_terms_url of this GenerativefirewallFirewallConfig.  # noqa: E501
        :rtype: str
        """
        return self._sensitive_terms_url

    @sensitive_terms_url.setter
    def sensitive_terms_url(self, sensitive_terms_url):
        """Sets the sensitive_terms_url of this GenerativefirewallFirewallConfig.

        Sensitive terms URL describes the location of a sensitive terms sheet to use in validation rules around PII. If not provided, the PII rules will not run.  # noqa: E501

        :param sensitive_terms_url: The sensitive_terms_url of this GenerativefirewallFirewallConfig.  # noqa: E501
        :type: str
        """

        self._sensitive_terms_url = sensitive_terms_url

    @property
    def topic(self):
        """Gets the topic of this GenerativefirewallFirewallConfig.  # noqa: E501

        Topic describes the expected chat topic. It is used for the Off-topic rule. Example: \"question answering about the US Bureau of Labor Statistics jobs report\" If not provided, the off-topic rule will not run.  # noqa: E501

        :return: The topic of this GenerativefirewallFirewallConfig.  # noqa: E501
        :rtype: str
        """
        return self._topic

    @topic.setter
    def topic(self, topic):
        """Sets the topic of this GenerativefirewallFirewallConfig.

        Topic describes the expected chat topic. It is used for the Off-topic rule. Example: \"question answering about the US Bureau of Labor Statistics jobs report\" If not provided, the off-topic rule will not run.  # noqa: E501

        :param topic: The topic of this GenerativefirewallFirewallConfig.  # noqa: E501
        :type: str
        """

        self._topic = topic

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GenerativefirewallFirewallConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GenerativefirewallFirewallConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RimeCreateFirewallRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'project_id': 'RimeUUID',
        'model_id': 'RimeUUID',
        'bin_size': 'str',
        'ref_data_id': 'str',
        'scheduled_ct_parameters': 'CreateFirewallRequestScheduledCTParameters'
    }

    attribute_map = {
        'project_id': 'projectId',
        'model_id': 'modelId',
        'bin_size': 'binSize',
        'ref_data_id': 'refDataId',
        'scheduled_ct_parameters': 'scheduledCtParameters'
    }

    def __init__(self, project_id=None, model_id=None, bin_size=None, ref_data_id=None, scheduled_ct_parameters=None):  # noqa: E501
        """RimeCreateFirewallRequest - a model defined in Swagger"""  # noqa: E501
        self._project_id = None
        self._model_id = None
        self._bin_size = None
        self._ref_data_id = None
        self._scheduled_ct_parameters = None
        self.discriminator = None
        if project_id is not None:
            self.project_id = project_id
        if model_id is not None:
            self.model_id = model_id
        self.bin_size = bin_size
        self.ref_data_id = ref_data_id
        if scheduled_ct_parameters is not None:
            self.scheduled_ct_parameters = scheduled_ct_parameters

    @property
    def project_id(self):
        """Gets the project_id of this RimeCreateFirewallRequest.  # noqa: E501


        :return: The project_id of this RimeCreateFirewallRequest.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this RimeCreateFirewallRequest.


        :param project_id: The project_id of this RimeCreateFirewallRequest.  # noqa: E501
        :type: RimeUUID
        """

        self._project_id = project_id

    @property
    def model_id(self):
        """Gets the model_id of this RimeCreateFirewallRequest.  # noqa: E501


        :return: The model_id of this RimeCreateFirewallRequest.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._model_id

    @model_id.setter
    def model_id(self, model_id):
        """Sets the model_id of this RimeCreateFirewallRequest.


        :param model_id: The model_id of this RimeCreateFirewallRequest.  # noqa: E501
        :type: RimeUUID
        """

        self._model_id = model_id

    @property
    def bin_size(self):
        """Gets the bin_size of this RimeCreateFirewallRequest.  # noqa: E501

        Duration of each bin size of continuous tests.  # noqa: E501

        :return: The bin_size of this RimeCreateFirewallRequest.  # noqa: E501
        :rtype: str
        """
        return self._bin_size

    @bin_size.setter
    def bin_size(self, bin_size):
        """Sets the bin_size of this RimeCreateFirewallRequest.

        Duration of each bin size of continuous tests.  # noqa: E501

        :param bin_size: The bin_size of this RimeCreateFirewallRequest.  # noqa: E501
        :type: str
        """
        if bin_size is None:
            raise ValueError("Invalid value for `bin_size`, must not be `None`")  # noqa: E501

        self._bin_size = bin_size

    @property
    def ref_data_id(self):
        """Gets the ref_data_id of this RimeCreateFirewallRequest.  # noqa: E501

        Uniquely specifies a reference data set.  # noqa: E501

        :return: The ref_data_id of this RimeCreateFirewallRequest.  # noqa: E501
        :rtype: str
        """
        return self._ref_data_id

    @ref_data_id.setter
    def ref_data_id(self, ref_data_id):
        """Sets the ref_data_id of this RimeCreateFirewallRequest.

        Uniquely specifies a reference data set.  # noqa: E501

        :param ref_data_id: The ref_data_id of this RimeCreateFirewallRequest.  # noqa: E501
        :type: str
        """
        if ref_data_id is None:
            raise ValueError("Invalid value for `ref_data_id`, must not be `None`")  # noqa: E501

        self._ref_data_id = ref_data_id

    @property
    def scheduled_ct_parameters(self):
        """Gets the scheduled_ct_parameters of this RimeCreateFirewallRequest.  # noqa: E501


        :return: The scheduled_ct_parameters of this RimeCreateFirewallRequest.  # noqa: E501
        :rtype: CreateFirewallRequestScheduledCTParameters
        """
        return self._scheduled_ct_parameters

    @scheduled_ct_parameters.setter
    def scheduled_ct_parameters(self, scheduled_ct_parameters):
        """Sets the scheduled_ct_parameters of this RimeCreateFirewallRequest.


        :param scheduled_ct_parameters: The scheduled_ct_parameters of this RimeCreateFirewallRequest.  # noqa: E501
        :type: CreateFirewallRequestScheduledCTParameters
        """

        self._scheduled_ct_parameters = scheduled_ct_parameters

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RimeCreateFirewallRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RimeCreateFirewallRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Library defining the interface to firewall."""
from datetime import timedelta
from typing import Any, Dict, Iterator, List, Optional

import pandas as pd
from deprecated import deprecated

from rime_sdk.internal.config_parser import (
    convert_single_data_info_to_swagger,
    convert_single_pred_info_to_swagger,
)
from rime_sdk.internal.constants import (
    MONITOR_TYPE_TO_SWAGGER,
    RISK_CATEGORY_TO_SWAGGER,
)
from rime_sdk.internal.decorators import prompt_confirmation
from rime_sdk.internal.rest_error_handler import RESTErrorHandler
from rime_sdk.internal.swagger_parser import parse_events_to_df
from rime_sdk.internal.swagger_utils import rest_to_timedelta, timedelta_to_rest
from rime_sdk.internal.throttle_queue import ThrottleQueue
from rime_sdk.internal.utils import convert_dict_to_html, make_link
from rime_sdk.job import ContinuousTestJob
from rime_sdk.monitor import Monitor
from rime_sdk.swagger import swagger_client
from rime_sdk.swagger.swagger_client import (
    ApiClient,
    ContinuoustestsFirewallIdUuidBody,
    FirewallFirewall,
    FirewallFirewallFirewallIdUuidBody,
    FirewallScheduledCTInfo,
    RimeUpdateFirewallResponse,
    RimeUUID,
    RuntimeinfoCustomImage,
    RuntimeinfoCustomImageType,
    RuntimeinfoResourceRequest,
    RuntimeinfoRunTimeInfo,
    TestrunTestRunIncrementalConfig,
    V1firewallfirewallFirewallIdUuidFirewall,
)
from rime_sdk.swagger.swagger_client.api.detection_api import DetectionApi
from rime_sdk.swagger.swagger_client.models import RimeJobMetadata
from rime_sdk.swagger.swagger_client.models.detection_event_type import (
    DetectionEventType,
)


@deprecated("Firewall class will not be supported from Robust Intelligence v2.4.")
class Firewall:
    """An interface to a RIME Firewall.

    **This class is deprecated in Robust Intelligence v2.2 and will be removed in v2.4.**
    """

    # A throttler that limits the number of model tests to roughly 20 every 5 minutes.
    # This is a static variable for Client.
    _throttler = ThrottleQueue(desired_events_per_epoch=20, epoch_duration_sec=300)

    def __init__(self, api_client: ApiClient, firewall_id: str) -> None:
        """Create a new Firewall object.

        Arguments:
            api_client: ApiClient
                The client used to query the RIME cluster.
            firewall_id: str
                The ID of the Firewall.
        """
        self._api_client = api_client
        self._firewall_id = firewall_id

    def __eq__(self, obj: Any) -> bool:
        """Check if this FWInstance is equivalent to 'obj'."""
        return isinstance(obj, Firewall) and self._firewall_id == obj._firewall_id

    def __repr__(self) -> str:
        """Return string representation of the object."""
        return f"Firewall({self._firewall_id})"

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def update_firewall(
        self,
        model_id: Optional[str] = None,
        ref_data_id: Optional[str] = None,
        scheduled_ct_eval_data_integration_id: Optional[str] = None,
        scheduled_ct_eval_data_info: Optional[dict] = None,
        scheduled_ct_eval_pred_integration_id: Optional[str] = None,
        scheduled_ct_eval_pred_info: Optional[dict] = None,
        update_rolling_window: bool = False,
        scheduled_ct_rolling_window_size: Optional[timedelta] = None,
        disable_scheduled_ct: Optional[bool] = False,
    ) -> RimeUpdateFirewallResponse:
        """Update the Firewall with specified model and reference data.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Arguments:
            model_id: Optional[str]
                The ID of the model to use for the Firewall.
            ref_data_id: Optional[str]
                The ID of the reference data to use for the Firewall.
            scheduled_ct_eval_data_integration_id: Optional[str]
                The integration id of the evaluation data for scheduled CT.
            scheduled_ct_eval_data_info: Optional[Dict]
                The data info of the evaluation data for scheduled CT.
            scheduled_ct_eval_pred_integration_id: Optional[str]
                The integration id of the evaluation prediction for scheduled CT.
            scheduled_ct_eval_pred_info: Optional[Dict]
                The data info of the evaluation prediction for scheduled CT.
            update_rolling_window: bool
                Specifies whether to update rolling window
            scheduled_ct_rolling_window_size: Optional[timedelta]
                If `update_rolling_window` is True, the new rolling window size
                will be set to this value. The rolling window for the firewall
                is used to determine the reference set for scheduled CT if it is
                not `None`.
            disable_scheduled_ct: Optional[bool]
                Specifies whether to disable continuous testing

        Returns:
            Dict:
                Dictionary representation of updated Firewall object.

        Raises:
            ValueError
                This error is generated when no fields are submitted to be updated or
                when the request to the Firewall service fails.

        Example:
            .. code-block:: python

                response = fw.update_firewall(ref_data_id="New reference data ID")
        """
        api = swagger_client.FirewallServiceApi(self._api_client)
        field_mask_list = []
        if model_id is not None:
            field_mask_list.append("modelId")
        if ref_data_id is not None:
            field_mask_list.append("refDataId")
        scheduled_ct_info = FirewallScheduledCTInfo()
        if scheduled_ct_eval_data_integration_id is not None:
            field_mask_list.append("scheduledCtInfo.evalDataIntegrationId")
            scheduled_ct_info.eval_data_integration_id = RimeUUID(
                uuid=scheduled_ct_eval_data_integration_id
            )
        if scheduled_ct_eval_data_info is not None:
            field_mask_list.append("scheduledCtInfo.evalDataInfo")
            scheduled_ct_info.eval_data_info = convert_single_data_info_to_swagger(
                scheduled_ct_eval_data_info
            )
        if scheduled_ct_eval_pred_integration_id is not None:
            field_mask_list.append("scheduledCtInfo.evalPredIntegrationId")
            scheduled_ct_info.eval_pred_integration_id = RimeUUID(
                uuid=scheduled_ct_eval_pred_integration_id
            )
        if scheduled_ct_eval_pred_info is not None:
            field_mask_list.append("scheduledCtInfo.evalPredInfo")
            scheduled_ct_info.eval_pred_info = convert_single_pred_info_to_swagger(
                scheduled_ct_eval_pred_info
            )
        if update_rolling_window:
            field_mask_list.append("scheduledCtInfo.rollingWindow")
            rolling_window = (
                timedelta_to_rest(scheduled_ct_rolling_window_size)
                if scheduled_ct_rolling_window_size
                else None
            )
            scheduled_ct_info.rolling_window = rolling_window
        if disable_scheduled_ct:
            field_mask_list.append("scheduledCtInfo.disableScheduledCt")
            scheduled_ct_info.disable_scheduled_ct = disable_scheduled_ct
        if len(field_mask_list) == 0:
            raise ValueError(
                "User must provide at least one of model_id, ref_data_id, "
                "or scheduled CT paramaters."
            )
        req = FirewallFirewallFirewallIdUuidBody(
            firewall_id=RimeUUID(self._firewall_id),
            firewall=V1firewallfirewallFirewallIdUuidFirewall(
                model_id=RimeUUID(model_id) if model_id is not None else None,
                ref_data_id=ref_data_id,
                scheduled_ct_info=scheduled_ct_info,
            ),
            mask=",".join(field_mask_list),
        )
        with RESTErrorHandler():
            resp = api.firewall_service_update_firewall(
                firewall_firewall_id_uuid=self._firewall_id,
                body=req,
            )
        print(f"Firewall {self._firewall_id} updated successfully.")
        return resp

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def activate_ct_scheduling(
        self,
        data_info: dict,
        data_integration_id: Optional[str] = None,
        pred_integration_id: Optional[str] = None,
        pred_info: Optional[dict] = None,
        rolling_window_size: Optional[timedelta] = None,
    ) -> None:
        """Activate scheduled CT.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Arguments:
            data_info: dict
                The data info of the evaluation data for scheduled CT.
            data_integration_id: Optional[str]
                The integration id of the evaluation data for scheduled CT.
            pred_integration_id: Optional[str]
                The integration id of the evaluation prediction for scheduled CT.
            pred_info: Optional[dict]
                The prediction info of the evaluation data for scheduled CT.
            rolling_window_size: Optional[timedelta]
                If `rolling_window_size` is not `None`, it is used to determine
                the reference set for scheduled CT.
        """
        if self.is_scheduled_ct_enabled():
            raise ValueError(
                "Scheduled CT is already enabled. Please use "
                "`update_scheduled_ct_info` to update the location info."
            )
        self.update_firewall(
            scheduled_ct_eval_data_integration_id=data_integration_id,
            scheduled_ct_eval_data_info=data_info,
            scheduled_ct_eval_pred_integration_id=pred_integration_id,
            scheduled_ct_eval_pred_info=pred_info,
            update_rolling_window=rolling_window_size is not None,
            scheduled_ct_rolling_window_size=rolling_window_size,
            disable_scheduled_ct=False,
        )
        print(f"Scheduled CT of Firewall {self._firewall_id} activated successfully.")

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def update_scheduled_ct_info(
        self,
        data_integration_id: Optional[str] = None,
        data_info: Optional[dict] = None,
        pred_integration_id: Optional[str] = None,
        pred_info: Optional[dict] = None,
    ) -> None:
        """Update scheduled CT.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Arguments:
            data_integration_id: Optional[str]
                If `data_integration_id` is not `None`, the Firewall will use it
                for scheduled CT.
            data_info: Optional[dict]
                If `data_info` is not `None`, the Firewall will use it for
                scheduled CT.
            pred_integration_id: Optional[str]
                If `pred_integration_id` is not `None`, the Firewall will use it
                for scheduled CT.
            pred_info: Optional[dict]
                If `pred_info` is not `None`, the Firewall will use it for scheduled CT.
        """
        self.update_firewall(
            scheduled_ct_eval_data_integration_id=data_integration_id,
            scheduled_ct_eval_data_info=data_info,
            scheduled_ct_eval_pred_integration_id=pred_integration_id,
            scheduled_ct_eval_pred_info=pred_info,
        )
        print(f"Location info of Firewall {self._firewall_id} updated successfully.")

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def deactivate_ct_scheduling(self) -> None:
        """Deactivate scheduled CT.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        """
        if not self.is_scheduled_ct_enabled():
            raise ValueError("Scheduled CT is already disabled.")
        self.update_firewall(disable_scheduled_ct=True)
        print(f"Scheduled CT of Firewall {self._firewall_id} deactivated successfully.")

    def _repr_html_(self) -> str:
        """Return HTML representation of the object."""
        info = {
            "Firewall ID": self._firewall_id,
            "Link": make_link(
                "https://" + self.get_link(),
                link_text="Continuous Testing Overview Page",
            ),
        }
        return convert_dict_to_html(info)

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def get_link(self) -> str:
        """Return a URL to the Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        api = swagger_client.FirewallServiceApi(self._api_client)
        with RESTErrorHandler():
            resp = api.firewall_service_get_url(
                firewall_id_uuid=self._firewall_id,
            )
        return resp.url.url

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def get_bin_size(self) -> timedelta:
        """Return the bin size of this Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        firewall = self._get_firewall()
        return rest_to_timedelta(firewall.bin_size)

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def get_ref_data_id(self) -> str:
        """Return the ID of the Firewall's current reference set.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        firewall = self._get_firewall()
        return firewall.ref_data_id

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def get_model_id(self) -> str:
        """Return the ID of the Firewall's current model.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        firewall = self._get_firewall()
        return firewall.model_id.uuid

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def get_scheduled_ct_info(self) -> FirewallScheduledCTInfo:
        """Return the scheduled continuous testing info of this Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        firewall = self._get_firewall()
        return firewall.scheduled_ct_info

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def is_scheduled_ct_enabled(self) -> bool:
        """Return whether scheduled continuous testing is enabled for this Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        firewall = self._get_firewall()
        is_enabled = firewall.scheduled_ct_info is not None
        is_enabled = is_enabled and not firewall.scheduled_ct_info.disable_scheduled_ct
        return is_enabled

    @property
    def project_id(self) -> str:
        """Return the ID of the parent project for this Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**
        """
        fw = self._get_firewall()
        return fw.project_id.uuid

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    @prompt_confirmation("Are you sure you want to delete this Firewall? (y/n) ")
    def delete_firewall(
        self,
        force: Optional[bool] = False,  # noqa: ARG002 (unused-method-argument)
    ) -> None:
        """Delete Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Args:
            force: Optional[bool] = False
                When set to True, the Firewall will be deleted immediately. By default,
                a confirmation is required.
        """
        api = swagger_client.FirewallServiceApi(self._api_client)
        with RESTErrorHandler():
            api.firewall_service_delete_firewall(firewall_id_uuid=self._firewall_id)
        print("Firewall successfully deleted.")

    def _get_firewall(self) -> FirewallFirewall:
        api = swagger_client.FirewallServiceApi(self._api_client)
        with RESTErrorHandler():
            res = api.firewall_service_get_firewall(firewall_id_uuid=self._firewall_id)
        return res.firewall

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def get_events_df(
        self,
    ) -> pd.DataFrame:
        """Get a dataframe of Detected Events for the given Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Monitors detect Events when degradations occur.
        For example, a Monitor for the metric "Accuracy" will detect an Event
        when the value of the model performance metric drops below a threshold.
        """
        detection_api = DetectionApi(self._api_client)
        next_page_token = ""
        has_more = True
        all_events = []
        while has_more:
            kwargs: Dict[str, Any] = {}
            if len(next_page_token) > 0:
                kwargs["page_token"] = next_page_token
            else:
                # Restrict to events for CT.
                kwargs["first_page_req_event_types"] = [
                    DetectionEventType.METRIC_DEGRADATION,
                    DetectionEventType.SECURITY,
                ]
            with RESTErrorHandler():
                res = detection_api.detection_list_detection_events(
                    self.project_id, **kwargs
                )
                next_page_token = res.next_page_token
                has_more = res.has_more
                all_events += res.events
        df = parse_events_to_df(all_events)
        return df

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def list_monitors(
        self,
        monitor_types: Optional[List[str]] = None,
        risk_category_types: Optional[List[str]] = None,
    ) -> Iterator[Monitor]:
        """List Monitors for the given Firewall.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Monitors examine time-sequenced data in RIME. Built-in Monitors track model
        health metrics such as degradations in model performance metrics or attacks
        on your model. This method can optionally filter by Monitor types or Risk
        Category types.

        Arguments:
            monitor_types: Optional[List[str]]
                Modifies query to return the set of built-in monitors or
                user-created custom monitors.
                Accepted values: ["Default", "Custom"]
            risk_category_types: Optional[List[str]]
                Modifies query to return monitors pertaining to certain categories
                of AI Risk. For instance, monitors that track model performance help
                you track down Operational Risk.
                Accepted values: \
                ["Operational", "Bias_and_Fairness", "Security", "Custom"]

        Returns:
            Iterator[Monitor]:
                A generator of Monitor objects.

        Raises:
            ValueError
                This error is generated when unrecognized filtering parameters are
                provided or when the request to the Firewall service fails.

        Example:
            .. code-block:: python

                # List all default Monitors on the Firewall
                monitors = firewall.list_monitors(monitor_types=["Default"])
                # For each Monitor, list all detected Events.
                all_events = [monitor.list_detected_events() for monitor in monitors]
        """
        swagger_monitor_types = []
        swagger_risk_types = []
        if monitor_types is not None:
            try:
                swagger_monitor_types = [
                    MONITOR_TYPE_TO_SWAGGER[m] for m in monitor_types
                ]
            except KeyError as e:
                raise ValueError(
                    f"{e.args[0]} is not a valid monitor type,"
                    + f" {list(MONITOR_TYPE_TO_SWAGGER.keys())}"
                    + " are the accepted monitor types."
                )
        if risk_category_types is not None:
            try:
                swagger_risk_types = [
                    RISK_CATEGORY_TO_SWAGGER[r] for r in risk_category_types
                ]
            except KeyError as e:
                raise ValueError(
                    f"{e.args[0]} is not a valid risk category type,"
                    + f" {list(RISK_CATEGORY_TO_SWAGGER.keys())}"
                    + " are the accepted risk category types."
                )
        project_id = self._get_firewall().project_id.uuid
        api = swagger_client.MonitorServiceApi(self._api_client)
        next_page_token = ""
        has_more = True
        while has_more:
            kwargs: Dict[str, Any] = {}
            if len(next_page_token) > 0:
                kwargs["page_token"] = next_page_token
            else:
                kwargs["first_page_req_included_monitor_types"] = swagger_monitor_types
                kwargs[
                    "first_page_req_included_risk_category_types"
                ] = swagger_risk_types
            with RESTErrorHandler():
                res = api.monitor_service_list_monitors(
                    firewall_id_uuid=self._firewall_id,
                    **kwargs,
                )
            for monitor in res.monitors:
                yield Monitor(
                    self._api_client, monitor.id.uuid, self._firewall_id, project_id
                )
            next_page_token = res.next_page_token
            has_more = res.has_more

    @deprecated(
        "Firewall class methods will not be supported from Robust Intelligence v2.4."
    )
    def start_continuous_test(
        self,
        eval_data_id: str,
        override_existing_bins: bool = False,
        agent_id: Optional[str] = None,
        ram_request_megabytes: Optional[int] = None,
        cpu_request_millicores: Optional[int] = None,
        random_seed: Optional[int] = None,
        rime_managed_image: Optional[str] = None,
        custom_image: Optional[RuntimeinfoCustomImage] = None,
        **exp_fields: Dict[str, object],
    ) -> ContinuousTestJob:
        """Start a Continuous Testing run.

        **Note that the Firewall is deprecated in Robust Intelligence v2.2 and will
        not be supported from v2.4.**

        Runs a Continuous Testing job on a batch of data.

        Arguments:
            eval_data_id: str
                ID of the evaluation data.
            override_existing_bins: bool
                Specifies whether to override existing bins.
            ram_request_megabytes: Optional[int]
                Megabytes of RAM set as the Kubernetes pod limit for the Stress Test
                Job. The default is 4000MB.
            cpu_request_millicores: Optional[int]
                Millicores of CPU set as the Kubernetes pod limit for the Stress Test
                Job. The default is 1500mi.
            random_seed: Optional[int]
                Random seed to use for the Job, so that Test Job result will be
                deterministic.
            agent_id: Optional[str]
                ID for the Agent where the Continuous Test will be run.
                Uses the default Agent for the workspace when not specified.
            rime_managed_image: Optional[str]
                Name of a Managed Image to use when running the model test.
                The image must have all dependencies required by your model. To create
                new Managed Images with your desired dependencies, use the client's
                `create_managed_image()` method.
            custom_image: Optional[RuntimeinfoCustomImage]
                Specification of a customized container image to use running the model
                test. The image must have all dependencies required by your model.
                The image must specify a name for the image and optionally a pull secret
                (of type RuntimeinfoCustomImagePullSecret) with the name of the
                kubernetes pull secret used to access the given image.
            exp_fields: Dict[str, object]
                Fields for experimental features.

        Returns:
            ContinuousTestJob:
                A ``Job`` object corresponding to the model Continuous Test Job.

        Raises:
            ValueError
                This error is generated when the request to the ModelTesting service
                fails.

        Example:
            .. code-block:: python

                firewall = project.get_firewall()
                eval_data_id = client.register_dataset("example dataset", data_config)
                job = firewall.start_continuous_test(
                    eval_data_id=eval_data_id,
                    ram_request_megabytes=8000,
                    cpu_request_millicores=2000,
                )
        """
        if ram_request_megabytes is not None and ram_request_megabytes <= 0:
            raise ValueError(
                "The requested number of megabytes of RAM must be positive"
            )

        if cpu_request_millicores is not None and cpu_request_millicores <= 0:
            raise ValueError(
                "The requested number of millicores of CPU must be positive"
            )

        req = ContinuoustestsFirewallIdUuidBody(
            test_run_incremental_config=TestrunTestRunIncrementalConfig(
                eval_dataset_id=eval_data_id,
                run_time_info=RuntimeinfoRunTimeInfo(
                    resource_request=RuntimeinfoResourceRequest(
                        ram_request_megabytes=ram_request_megabytes,
                        cpu_request_millicores=cpu_request_millicores,
                    ),
                    random_seed=random_seed,
                ),
            ),
            override_existing_bins=override_existing_bins,
            experimental_fields=exp_fields if exp_fields else None,
            agent_id=RimeUUID(agent_id) if agent_id else None,
        )
        if custom_image:
            req.test_run_incremental_config.run_time_info.custom_image = (
                RuntimeinfoCustomImageType(custom_image=custom_image)
            )
        if rime_managed_image:
            req.test_run_incremental_config.run_time_info.custom_image = (
                RuntimeinfoCustomImageType(managed_image_name=rime_managed_image)
            )
        with RESTErrorHandler():
            Firewall._throttler.throttle(
                throttling_msg="Your request is throttled to limit # of model tests."
            )
            api = swagger_client.ModelTestingApi(self._api_client)
            job: RimeJobMetadata = api.model_testing_start_continuous_test(
                body=req,
                firewall_id_uuid=self._firewall_id,
            ).job
        return ContinuousTestJob(self._api_client, job.job_id)



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class UserUserDetail(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'RimeUUID',
        'name': 'str',
        'role': 'UserRole',
        'email': 'str',
        'full_name': 'str',
        'show_tutorial': 'bool',
        'org_role': 'RimeActorRole',
        'private_info': 'UserPrivateInfo'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'role': 'role',
        'email': 'email',
        'full_name': 'fullName',
        'show_tutorial': 'showTutorial',
        'org_role': 'orgRole',
        'private_info': 'privateInfo'
    }

    def __init__(self, id=None, name=None, role=None, email=None, full_name=None, show_tutorial=None, org_role=None, private_info=None):  # noqa: E501
        """UserUserDetail - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._name = None
        self._role = None
        self._email = None
        self._full_name = None
        self._show_tutorial = None
        self._org_role = None
        self._private_info = None
        self.discriminator = None
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if role is not None:
            self.role = role
        if email is not None:
            self.email = email
        if full_name is not None:
            self.full_name = full_name
        if show_tutorial is not None:
            self.show_tutorial = show_tutorial
        if org_role is not None:
            self.org_role = org_role
        if private_info is not None:
            self.private_info = private_info

    @property
    def id(self):
        """Gets the id of this UserUserDetail.  # noqa: E501


        :return: The id of this UserUserDetail.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this UserUserDetail.


        :param id: The id of this UserUserDetail.  # noqa: E501
        :type: RimeUUID
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this UserUserDetail.  # noqa: E501

        Name of the user.  # noqa: E501

        :return: The name of this UserUserDetail.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this UserUserDetail.

        Name of the user.  # noqa: E501

        :param name: The name of this UserUserDetail.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def role(self):
        """Gets the role of this UserUserDetail.  # noqa: E501


        :return: The role of this UserUserDetail.  # noqa: E501
        :rtype: UserRole
        """
        return self._role

    @role.setter
    def role(self, role):
        """Sets the role of this UserUserDetail.


        :param role: The role of this UserUserDetail.  # noqa: E501
        :type: UserRole
        """

        self._role = role

    @property
    def email(self):
        """Gets the email of this UserUserDetail.  # noqa: E501


        :return: The email of this UserUserDetail.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this UserUserDetail.


        :param email: The email of this UserUserDetail.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def full_name(self):
        """Gets the full_name of this UserUserDetail.  # noqa: E501


        :return: The full_name of this UserUserDetail.  # noqa: E501
        :rtype: str
        """
        return self._full_name

    @full_name.setter
    def full_name(self, full_name):
        """Sets the full_name of this UserUserDetail.


        :param full_name: The full_name of this UserUserDetail.  # noqa: E501
        :type: str
        """

        self._full_name = full_name

    @property
    def show_tutorial(self):
        """Gets the show_tutorial of this UserUserDetail.  # noqa: E501


        :return: The show_tutorial of this UserUserDetail.  # noqa: E501
        :rtype: bool
        """
        return self._show_tutorial

    @show_tutorial.setter
    def show_tutorial(self, show_tutorial):
        """Sets the show_tutorial of this UserUserDetail.


        :param show_tutorial: The show_tutorial of this UserUserDetail.  # noqa: E501
        :type: bool
        """

        self._show_tutorial = show_tutorial

    @property
    def org_role(self):
        """Gets the org_role of this UserUserDetail.  # noqa: E501


        :return: The org_role of this UserUserDetail.  # noqa: E501
        :rtype: RimeActorRole
        """
        return self._org_role

    @org_role.setter
    def org_role(self, org_role):
        """Sets the org_role of this UserUserDetail.


        :param org_role: The org_role of this UserUserDetail.  # noqa: E501
        :type: RimeActorRole
        """

        self._org_role = org_role

    @property
    def private_info(self):
        """Gets the private_info of this UserUserDetail.  # noqa: E501


        :return: The private_info of this UserUserDetail.  # noqa: E501
        :rtype: UserPrivateInfo
        """
        return self._private_info

    @private_info.setter
    def private_info(self, private_info):
        """Sets the private_info of this UserUserDetail.


        :param private_info: The private_info of this UserUserDetail.  # noqa: E501
        :type: UserPrivateInfo
        """

        self._private_info = private_info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(UserUserDetail, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UserUserDetail):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Library for adaptively throttling events."""

import logging
import time

logger = logging.getLogger(__name__)


class ThrottleQueue:
    """A throttle queue is used to regulate the rate of events.

    The throttle queue queues a history of past events in order to
    moderate the rate at which expensive events occur.
    """

    def __init__(
        self, desired_events_per_epoch: int, epoch_duration_sec: float
    ) -> None:
        """Create a new throttle queue for a given desired event rate.

        Args:
            desired_events_per_epoch: int
                The number events that are desired for the given epoch.
            epoch_duration_sec: float
                The lenth of the epoch in seconds.
        """
        # A circular queue of past event timestamps that is 3 times the size of
        # the number of desired events.
        self._timestamp_queue = [0.0] * (3 * desired_events_per_epoch)
        self._timestamp_index = 0

        self._desired_events_per_epoch = desired_events_per_epoch
        self._epoch_duration_sec = epoch_duration_sec
        self._desired_interval = (
            self._epoch_duration_sec / self._desired_events_per_epoch
        )

    def throttle(self, throttling_msg: str) -> bool:
        """Register a new event and throttles it according to the past events.

        Returns:
            Whether or not this event was throttled.
        """
        now = time.time()
        # Compute the number of events in the current epoch above the desired number.
        events_in_epoch = sum(
            [now - x < self._epoch_duration_sec for x in self._timestamp_queue]
        )
        excess_events = events_in_epoch - self._desired_events_per_epoch
        throttled = excess_events >= 0
        if throttled:
            logger.warning(f"Throttling: {throttling_msg}")

        # Add an exponential sleep penalty to throttle the events toward the desired
        # interval.
        # Note: small amounts of throttling occur while excess_events < 0. The total
        # contribution of this delay is
        #    T = DI * sum_{t=1}{N} 2^-t
        #      < DI * sum_{t=1}{inf} 2^-t
        #      = DI
        # where DI is the desired interval and N is the desired events per epoch.
        # Thus, there will be at most 1 unit of desired interval in delays before
        # full throttling is applied.
        time.sleep(self._desired_interval * 2.0**excess_events)

        # Record the latest event in the circular queue.
        self._timestamp_queue[self._timestamp_index] = time.time()
        self._timestamp_index = (self._timestamp_index + 1) % len(self._timestamp_queue)
        return throttled



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DataParamsRankingInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'query_col': 'str',
        'nqueries': 'str',
        'nrows_per_query': 'str',
        'drop_query_id': 'bool'
    }

    attribute_map = {
        'query_col': 'queryCol',
        'nqueries': 'nqueries',
        'nrows_per_query': 'nrowsPerQuery',
        'drop_query_id': 'dropQueryId'
    }

    def __init__(self, query_col=None, nqueries=None, nrows_per_query=None, drop_query_id=None):  # noqa: E501
        """DataParamsRankingInfo - a model defined in Swagger"""  # noqa: E501
        self._query_col = None
        self._nqueries = None
        self._nrows_per_query = None
        self._drop_query_id = None
        self.discriminator = None
        if query_col is not None:
            self.query_col = query_col
        if nqueries is not None:
            self.nqueries = nqueries
        if nrows_per_query is not None:
            self.nrows_per_query = nrows_per_query
        if drop_query_id is not None:
            self.drop_query_id = drop_query_id

    @property
    def query_col(self):
        """Gets the query_col of this DataParamsRankingInfo.  # noqa: E501

        Name of column in dataset that contains the query ids.  # noqa: E501

        :return: The query_col of this DataParamsRankingInfo.  # noqa: E501
        :rtype: str
        """
        return self._query_col

    @query_col.setter
    def query_col(self, query_col):
        """Sets the query_col of this DataParamsRankingInfo.

        Name of column in dataset that contains the query ids.  # noqa: E501

        :param query_col: The query_col of this DataParamsRankingInfo.  # noqa: E501
        :type: str
        """

        self._query_col = query_col

    @property
    def nqueries(self):
        """Gets the nqueries of this DataParamsRankingInfo.  # noqa: E501

        Number of queries to consider. If null, will use all queries.  # noqa: E501

        :return: The nqueries of this DataParamsRankingInfo.  # noqa: E501
        :rtype: str
        """
        return self._nqueries

    @nqueries.setter
    def nqueries(self, nqueries):
        """Sets the nqueries of this DataParamsRankingInfo.

        Number of queries to consider. If null, will use all queries.  # noqa: E501

        :param nqueries: The nqueries of this DataParamsRankingInfo.  # noqa: E501
        :type: str
        """

        self._nqueries = nqueries

    @property
    def nrows_per_query(self):
        """Gets the nrows_per_query of this DataParamsRankingInfo.  # noqa: E501

        Number of rows to use per query. If null, will use all rows.  # noqa: E501

        :return: The nrows_per_query of this DataParamsRankingInfo.  # noqa: E501
        :rtype: str
        """
        return self._nrows_per_query

    @nrows_per_query.setter
    def nrows_per_query(self, nrows_per_query):
        """Sets the nrows_per_query of this DataParamsRankingInfo.

        Number of rows to use per query. If null, will use all rows.  # noqa: E501

        :param nrows_per_query: The nrows_per_query of this DataParamsRankingInfo.  # noqa: E501
        :type: str
        """

        self._nrows_per_query = nrows_per_query

    @property
    def drop_query_id(self):
        """Gets the drop_query_id of this DataParamsRankingInfo.  # noqa: E501

        Whether to drop the query ID column from the dataset to avoid passing as a feature to the model.  # noqa: E501

        :return: The drop_query_id of this DataParamsRankingInfo.  # noqa: E501
        :rtype: bool
        """
        return self._drop_query_id

    @drop_query_id.setter
    def drop_query_id(self, drop_query_id):
        """Sets the drop_query_id of this DataParamsRankingInfo.

        Whether to drop the query ID column from the dataset to avoid passing as a feature to the model.  # noqa: E501

        :param drop_query_id: The drop_query_id of this DataParamsRankingInfo.  # noqa: E501
        :type: bool
        """

        self._drop_query_id = drop_query_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DataParamsRankingInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DataParamsRankingInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RegistryConnectionInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'data_file': 'RegistryDataFileInfo',
        'data_loading': 'RegistryDataLoadingInfo',
        'data_collector': 'RegistryDataCollectorInfo',
        'databricks': 'RegistryDatabricksInfo',
        'hugging_face': 'RegistryHuggingFaceDataInfo'
    }

    attribute_map = {
        'data_file': 'dataFile',
        'data_loading': 'dataLoading',
        'data_collector': 'dataCollector',
        'databricks': 'databricks',
        'hugging_face': 'huggingFace'
    }

    def __init__(self, data_file=None, data_loading=None, data_collector=None, databricks=None, hugging_face=None):  # noqa: E501
        """RegistryConnectionInfo - a model defined in Swagger"""  # noqa: E501
        self._data_file = None
        self._data_loading = None
        self._data_collector = None
        self._databricks = None
        self._hugging_face = None
        self.discriminator = None
        if data_file is not None:
            self.data_file = data_file
        if data_loading is not None:
            self.data_loading = data_loading
        if data_collector is not None:
            self.data_collector = data_collector
        if databricks is not None:
            self.databricks = databricks
        if hugging_face is not None:
            self.hugging_face = hugging_face

    @property
    def data_file(self):
        """Gets the data_file of this RegistryConnectionInfo.  # noqa: E501


        :return: The data_file of this RegistryConnectionInfo.  # noqa: E501
        :rtype: RegistryDataFileInfo
        """
        return self._data_file

    @data_file.setter
    def data_file(self, data_file):
        """Sets the data_file of this RegistryConnectionInfo.


        :param data_file: The data_file of this RegistryConnectionInfo.  # noqa: E501
        :type: RegistryDataFileInfo
        """

        self._data_file = data_file

    @property
    def data_loading(self):
        """Gets the data_loading of this RegistryConnectionInfo.  # noqa: E501


        :return: The data_loading of this RegistryConnectionInfo.  # noqa: E501
        :rtype: RegistryDataLoadingInfo
        """
        return self._data_loading

    @data_loading.setter
    def data_loading(self, data_loading):
        """Sets the data_loading of this RegistryConnectionInfo.


        :param data_loading: The data_loading of this RegistryConnectionInfo.  # noqa: E501
        :type: RegistryDataLoadingInfo
        """

        self._data_loading = data_loading

    @property
    def data_collector(self):
        """Gets the data_collector of this RegistryConnectionInfo.  # noqa: E501


        :return: The data_collector of this RegistryConnectionInfo.  # noqa: E501
        :rtype: RegistryDataCollectorInfo
        """
        return self._data_collector

    @data_collector.setter
    def data_collector(self, data_collector):
        """Sets the data_collector of this RegistryConnectionInfo.


        :param data_collector: The data_collector of this RegistryConnectionInfo.  # noqa: E501
        :type: RegistryDataCollectorInfo
        """

        self._data_collector = data_collector

    @property
    def databricks(self):
        """Gets the databricks of this RegistryConnectionInfo.  # noqa: E501


        :return: The databricks of this RegistryConnectionInfo.  # noqa: E501
        :rtype: RegistryDatabricksInfo
        """
        return self._databricks

    @databricks.setter
    def databricks(self, databricks):
        """Sets the databricks of this RegistryConnectionInfo.


        :param databricks: The databricks of this RegistryConnectionInfo.  # noqa: E501
        :type: RegistryDatabricksInfo
        """

        self._databricks = databricks

    @property
    def hugging_face(self):
        """Gets the hugging_face of this RegistryConnectionInfo.  # noqa: E501


        :return: The hugging_face of this RegistryConnectionInfo.  # noqa: E501
        :rtype: RegistryHuggingFaceDataInfo
        """
        return self._hugging_face

    @hugging_face.setter
    def hugging_face(self, hugging_face):
        """Sets the hugging_face of this RegistryConnectionInfo.


        :param hugging_face: The hugging_face of this RegistryConnectionInfo.  # noqa: E501
        :type: RegistryHuggingFaceDataInfo
        """

        self._hugging_face = hugging_face

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RegistryConnectionInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RegistryConnectionInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DatacollectorDatapointRow(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'input_data': 'str',
        'label': 'str',
        'timestamp': 'datetime',
        'query_id': 'str'
    }

    attribute_map = {
        'input_data': 'inputData',
        'label': 'label',
        'timestamp': 'timestamp',
        'query_id': 'queryId'
    }

    def __init__(self, input_data=None, label=None, timestamp=None, query_id=None):  # noqa: E501
        """DatacollectorDatapointRow - a model defined in Swagger"""  # noqa: E501
        self._input_data = None
        self._label = None
        self._timestamp = None
        self._query_id = None
        self.discriminator = None
        if input_data is not None:
            self.input_data = input_data
        if label is not None:
            self.label = label
        if timestamp is not None:
            self.timestamp = timestamp
        if query_id is not None:
            self.query_id = query_id

    @property
    def input_data(self):
        """Gets the input_data of this DatacollectorDatapointRow.  # noqa: E501


        :return: The input_data of this DatacollectorDatapointRow.  # noqa: E501
        :rtype: str
        """
        return self._input_data

    @input_data.setter
    def input_data(self, input_data):
        """Sets the input_data of this DatacollectorDatapointRow.


        :param input_data: The input_data of this DatacollectorDatapointRow.  # noqa: E501
        :type: str
        """

        self._input_data = input_data

    @property
    def label(self):
        """Gets the label of this DatacollectorDatapointRow.  # noqa: E501


        :return: The label of this DatacollectorDatapointRow.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this DatacollectorDatapointRow.


        :param label: The label of this DatacollectorDatapointRow.  # noqa: E501
        :type: str
        """

        self._label = label

    @property
    def timestamp(self):
        """Gets the timestamp of this DatacollectorDatapointRow.  # noqa: E501

        If timestamp is not provided in request, the time when the datapoint is stored will be used.  # noqa: E501

        :return: The timestamp of this DatacollectorDatapointRow.  # noqa: E501
        :rtype: datetime
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """Sets the timestamp of this DatacollectorDatapointRow.

        If timestamp is not provided in request, the time when the datapoint is stored will be used.  # noqa: E501

        :param timestamp: The timestamp of this DatacollectorDatapointRow.  # noqa: E501
        :type: datetime
        """

        self._timestamp = timestamp

    @property
    def query_id(self):
        """Gets the query_id of this DatacollectorDatapointRow.  # noqa: E501

        Query ID is required for Ranking model tasks.  # noqa: E501

        :return: The query_id of this DatacollectorDatapointRow.  # noqa: E501
        :rtype: str
        """
        return self._query_id

    @query_id.setter
    def query_id(self, query_id):
        """Sets the query_id of this DatacollectorDatapointRow.

        Query ID is required for Ranking model tasks.  # noqa: E501

        :param query_id: The query_id of this DatacollectorDatapointRow.  # noqa: E501
        :type: str
        """

        self._query_id = query_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DatacollectorDatapointRow, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DatacollectorDatapointRow):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Library that defines the interface to the Registry."""
import json
import logging
from http import HTTPStatus
from typing import Dict, Iterator, List, Optional, cast

from rime_sdk.internal.config_parser import (
    convert_model_info_to_swagger,
    convert_single_data_info_to_swagger,
    convert_single_pred_info_to_swagger,
)
from rime_sdk.internal.rest_error_handler import RESTErrorHandler
from rime_sdk.internal.swagger_utils import serialize_datetime_to_proto_timestamp
from rime_sdk.swagger import swagger_client
from rime_sdk.swagger.swagger_client import (
    ApiClient,
    DatasetCTDatasetType,
    DatasetCTInfo,
)
from rime_sdk.swagger.swagger_client.models import (
    DatasetIdPredictionBody,
    ProjectIdUuidDatasetBody,
    ProjectIdUuidModelBody,
    RegistryMetadata,
    RimeGetDatasetResponse,
    RimeGetModelResponse,
    RimeGetPredictionSetResponse,
    RimeListDatasetsResponse,
    RimeListModelsResponse,
    RimeListPredictionSetsResponse,
    RimeRegisterDatasetResponse,
    RimeRegisterModelResponse,
    RimeUUID,
)
from rime_sdk.swagger.swagger_client.rest import ApiException

logger = logging.getLogger(__name__)


class Registry:
    """An interface to a RIME Registry."""

    def __init__(self, api_client: ApiClient) -> None:
        """Create a new Registry object.

        Arguments:
            api_client: ApiClient
                The client used to query the RIME cluster.
        """
        self._api_client = api_client

    def register_dataset(
        self,
        project_id: str,
        name: str,
        data_config: dict,
        integration_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        metadata: Optional[dict] = None,
        ct_info: Optional[dict] = None,
        skip_validation: Optional[bool] = False,
        agent_id: Optional[str] = None,
    ) -> str:
        """Register and validate a new dataset in a Project.

        Args:
            project_id: str
                The ID of the Project in which to register the dataset.
            name: str
                The chosen name of the dataset.
            data_config: dict
                A dictionary that contains the data configuration.
                The data configuration must match the API specification
                of the `data_info` field in the `RegisterDataset` request.
            integration_id: Optional[str] = None,
                Provide the integration ID for datasets that require an integration.
            tags: Optional[List[str]] = None,
                An optional list of tags to associate with the dataset.
            metadata: Optional[dict] = None,
                An optional dictionary of metadata to associate with the dataset.
            ct_info: Optional[dict] = None,
                An optional dictionary that contains the CT info.
                The CT info must match the API specification of the `ct_info`
                field in the `RegisterDataset` request.
            skip_validation: Optional[bool] = False,
                An optional boolean that indicates whether to skip validation.
                Validation ensures that the dataset is valid for Robust
                Intelligence's systems.
            agent_id: Optional[str] = None,
                Agent for running validation. If omitted the workspace's default
                agent will be used.

        Returns:
            str:
                The ID of the newly registered dataset.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.

        Example:
            .. code-block:: python

                dataset_id = registry.register_dataset(
                    name=DATASET_NAME,
                    data_config={
                        "connection_info": {"data_file": {"path": FILE_PATH}},
                        "data_params": {"label_col": LABEL_COL},
                    },
                    integration_id=INTEGRATION_ID,
                )
        """
        data_info_swagger = convert_single_data_info_to_swagger(data_config)
        req = ProjectIdUuidDatasetBody(
            project_id=RimeUUID(uuid=project_id),
            name=name,
            data_info=data_info_swagger,
            skip_validation=skip_validation,
            agent_id=RimeUUID(agent_id) if agent_id else None,
        )

        metadata_str: Optional[str] = None
        if metadata is not None:
            metadata_str = json.dumps(metadata)
        if tags is not None or metadata_str is not None:
            req.metadata = RegistryMetadata(tags=tags, extra_info=metadata_str)

        if integration_id is not None:
            req.integration_id = RimeUUID(uuid=integration_id)

        if ct_info is not None:
            req.ct_info = DatasetCTInfo(
                firewall_id=RimeUUID(uuid=ct_info["firewall_id"]),
                start_time=serialize_datetime_to_proto_timestamp(ct_info["start_time"]),
                end_time=serialize_datetime_to_proto_timestamp(ct_info["end_time"]),
                ct_dataset_type=DatasetCTDatasetType.USER_SPECIFIED,
            )

        with RESTErrorHandler():
            api = swagger_client.RegistryServiceApi(self._api_client)
            res = api.registry_service_register_dataset(
                body=req,
                project_id_uuid=project_id,
            )

            res = cast(RimeRegisterDatasetResponse, res)

        return res.dataset_id

    def register_model(
        self,
        project_id: str,
        name: str,
        model_config: Optional[dict] = None,
        tags: Optional[List[str]] = None,
        metadata: Optional[dict] = None,
        external_id: Optional[str] = None,
        integration_id: Optional[str] = None,
        model_endpoint_integration_id: Optional[str] = None,
        skip_validation: Optional[bool] = False,
        agent_id: Optional[str] = None,
    ) -> str:
        """Register and validate a new model in a Project.

        Args:
            project_id: str
                The ID of the Project in which to register the model.
            name: str
                The chosen name of the model.
            model_config: Optional[dict] = None,
                A dictionary that contains the model configuration.
                Any model configuration that is provided must match the API
                specification for the `model_info` field of the `RegisterModel`
                request.
            tags: Optional[List[str]] = None,
                An optional list of tags to associate with the model.
            metadata: Optional[dict] = None,
                An optional dictionary of metadata to associate with the model.
            external_id: Optional[str] = None,
                An optional external ID that can be used to identify the model.
            integration_id: Optional[str] = None,
                Provide the integration ID for models that require an
                integration for access.
            model_endpoint_integration_id: Optional[str] = None,
                Provide the integration ID for models that require an
                integration when running the model.
            skip_validation: Optional[bool] = False,
                An optional boolean that indicates whether to skip validation.
                Validation ensures that the model is valid for Robust
                Intelligence's systems.
            agent_id: Optional[str] = None,
                Agent for running validation. If omitted the workspace's default
                agent will be used.

        Returns:
            str:
                The ID of the newly registered model.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.

        Example:
            .. code-block:: python

                model_id = registry.register_model(
                    name=MODEL_NAME,
                    model_config={
                        "hugging_face": {
                            "model_uri": URI,
                            "kwargs": {
                                "tokenizer_uri": TOKENIZER_URI,
                                "class_map": MAP,
                                "ignore_class_names": True,
                            },
                        }
                    },
                    tags=[MODEL_TAG],
                    metadata={KEY: VALUE},
                    external_id=EXTERNAL_ID,
                    agent_id=AGENT_ID,
                )
        """
        req = ProjectIdUuidModelBody(
            project_id=RimeUUID(uuid=project_id),
            name=name,
            skip_validation=skip_validation,
            agent_id=RimeUUID(agent_id) if agent_id else None,
        )

        if model_config is not None:
            # When the `model_path` key is provided to the dictionary, the value
            # must be a dictionary whose `path` value points to a python
            # file that holds a `predict_dict` or `predict_df` function.
            # When the `model_loading` key is provided to the dictionary, the value
            # must be a dictionary whose `path` value points to a python
            # file that holds a `get_predict_df` or `get_predict_dict` function and
            # whose value for the `params` key is a dictionary of parameters to
            # pass to the function.
            # When the `hugging_face` key is provided to the dictionary, the value must
            # be a dictionary whose `model_uri` value points to a
            # hugging face model and whose value for the `params` key is a dictionary
            # of parameters that hugging face model requires.
            model_info = convert_model_info_to_swagger(model_config)
            req.model_info = model_info

        metadata_str: Optional[str] = None
        if metadata:
            metadata_str = json.dumps(metadata)
        if tags or metadata_str:
            req.metadata = RegistryMetadata(tags=tags, extra_info=metadata_str)
        if external_id:
            req.external_id = external_id
        if integration_id is not None:
            req.integration_id = RimeUUID(uuid=integration_id)
        if model_endpoint_integration_id is not None:
            req.model_endpoint_integration_id = RimeUUID(
                uuid=model_endpoint_integration_id
            )

        with RESTErrorHandler():
            api = swagger_client.RegistryServiceApi(self._api_client)
            res = api.registry_service_register_model(
                body=req,
                project_id_uuid=project_id,
            )

            res = cast(RimeRegisterModelResponse, res)

        return cast(RimeUUID, res.model_id).uuid

    def register_predictions(
        self,
        project_id: str,
        dataset_id: str,
        model_id: str,
        pred_config: dict,
        integration_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        metadata: Optional[dict] = None,
        skip_validation: Optional[bool] = False,
        agent_id: Optional[str] = None,
    ) -> None:
        """Register and validate a new set of predictions for a model and a dataset.

        Args:
            project_id: str
                The ID of the Project to which the models belong.
            dataset_id: str,
                The ID of the dataset used to generate the predictions.
            model_id: str,
                The ID of the model used to generate the predictions.
            pred_config: dict,
                A dictionary that contains the prediction configuration.
                The prediction configuration must match the API specification
                for the `pred_info` field of the `RegisterPredictions` request.
            integration_id: Optional[str] = None,
                Provide the integration ID for predictions that require an
                integration to use.
            tags: Optional[List[str]] = None,
                An optional list of tags to associate with the predictions.
            metadata: Optional[dict] = None,
                An optional dictionary of metadata to associate with the predictions.
            skip_validation: Optional[bool] = False,
                An optional boolean that indicates whether to skip validation.
                Validation ensures that the predictions are valid for Robust
                Intelligence's systems.
            agent_id: Optional[str] = None,
                Agent for running validation. If omitted the workspace's default
                agent will be used.

        Returns:
            None

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.

        Example:
            .. code-block:: python

                registry.register_predictions(
                    dataset_id=DATASET_ID,
                    model_id=MODEL_ID,
                    pred_config={
                        "connection_info": {
                            "databricks": {
                                # Unix timestamp equivalent to 02/08/2023
                                "start_time": 1675922943,
                                # Unix timestamp equivalent to 03/08/2023
                                "end_time": 1678342145,
                                "table_name": TABLE_NAME,
                                "time_col": TIME_COL,
                            },
                        },
                        "pred_params": {"pred_col": PREDS},
                    },
                    tags=[TAG],
                    metadata={KEY: VALUE},
                )
        """
        pred_info_swagger = convert_single_pred_info_to_swagger(pred_config)

        req = DatasetIdPredictionBody(
            project_id=RimeUUID(uuid=project_id),
            model_id=RimeUUID(uuid=model_id),
            pred_info=pred_info_swagger,
            skip_validation=skip_validation,
            agent_id=RimeUUID(agent_id) if agent_id else None,
        )

        metadata_str: Optional[str] = None
        if metadata is not None:
            metadata_str = json.dumps(metadata)
        if tags is not None or metadata_str is not None:
            req.metadata = RegistryMetadata(tags=tags, extra_info=metadata_str)

        if integration_id is not None:
            req.integration_id = RimeUUID(uuid=integration_id)

        with RESTErrorHandler():
            api = swagger_client.RegistryServiceApi(self._api_client)
            _ = api.registry_service_register_prediction_set(
                body=req,
                project_id_uuid=project_id,
                model_id_uuid=model_id,
                dataset_id=dataset_id,
            )

    def list_datasets(self, project_id: str) -> Iterator[Dict]:
        """Return a list of datasets.

        Args:
            project_id: str
                The ID of the Project to which the datasets belong.

        Returns:
            Iterator[Dict]:
                Iterator of dictionaries: each dictionary represents a
                dataset.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        # Iterate through the pages of datasets and break at the last page.
        page_token = ""
        with RESTErrorHandler():
            while True:
                if page_token == "":
                    res: RimeListDatasetsResponse = api.registry_service_list_datasets(
                        project_id_uuid=project_id
                    )
                else:
                    res = api.registry_service_list_datasets(
                        project_id_uuid=project_id, page_token=page_token
                    )
                if res.datasets is not None:
                    for dataset in res.datasets:
                        yield dataset.to_dict()
                # Advance to the next page of datasets.
                page_token = res.next_page_token
                # we've reached the last page of datasets.
                if not res.has_more:
                    break

    def list_models(self, project_id: str) -> Iterator[Dict]:
        """Return a list of models.

        Args:
            project_id: str
                The ID of the Project to which the models belong.

        Returns:
            Iterator[Dict]:
                Iterator of dictionaries: each dictionary represents a
                model.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        # Iterate through the pages of datasets and break at the last page.
        page_token = ""
        with RESTErrorHandler():
            while True:
                if page_token == "":
                    res: RimeListModelsResponse = api.registry_service_list_models(
                        project_id_uuid=project_id
                    )
                else:
                    res = api.registry_service_list_models(
                        project_id_uuid=project_id, page_token=page_token
                    )
                if res.models is not None:
                    for model in res.models:
                        yield model.model.to_dict()
                # Advance to the next page of models.
                page_token = res.next_page_token
                # we've reached the last page of models.
                if not res.has_more:
                    break

    def list_predictions(
        self,
        project_id: str,
        model_id: Optional[str] = None,
        dataset_id: Optional[str] = None,
    ) -> Iterator[Dict]:
        """Return a list of prediction sets.

        Args:
            project_id: str
                The ID of the Project to which the models belong.
            model_id: Optional[str] = None
                The ID of the model to which the prediction sets belong.
            dataset_id: Optional[str] = None
                The ID of the dataset to which the prediction sets belong.

        Returns:
            Iterator[Dict]:
                Iterator of dictionaries: each dictionary represents a
                prediction set.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        if model_id is not None and dataset_id is not None:
            raise ValueError("Only one of model_id or dataset_id can be set.")
        if model_id is None and dataset_id is None:
            raise ValueError("Must specify either a model_id or dataset_id.")
        else:
            pass
        api = swagger_client.RegistryServiceApi(self._api_client)
        # Iterate through the pages of datasets and break at the last page.
        page_token = ""
        with RESTErrorHandler():
            while True:
                if page_token == "":
                    if model_id is not None:
                        res: RimeListPredictionSetsResponse = (
                            api.registry_service_list_prediction_sets(
                                project_id_uuid=project_id,
                                first_page_req_model_id=model_id,
                            )
                        )
                    else:
                        res = api.registry_service_list_prediction_sets(
                            project_id_uuid=project_id,
                            first_page_req_dataset_id=dataset_id,
                        )
                else:
                    res = api.registry_service_list_prediction_sets(
                        project_id_uuid=project_id,
                        page_token=page_token,
                    )
                if res.predictions is not None:
                    for prediction in res.predictions:
                        yield prediction.to_dict()
                # Advance to the next page of predictions.
                page_token = res.next_page_token
                # we've reached the last page of predictions.
                if not res.has_more:
                    break

    def get_dataset(
        self, dataset_id: Optional[str] = None, dataset_name: Optional[str] = None
    ) -> Dict:
        """Return a dataset.

        Args:
            dataset_id: str
                The ID of the dataset to retrieve.
            dataset_name: str
                The name of the dataset to retrieve.

        Returns:
            Dict:
                A dictionary representing the dataset.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        if dataset_id is not None and dataset_name is not None:
            raise ValueError(
                "Arguments dataset_id and dataset_name"
                " are mutually exclusive.\n"
                f"Received dataset_id={dataset_id}"
                f" and dataset_name={dataset_name}."
            )
        elif dataset_name is None and dataset_id is None:
            raise ValueError("Must provide either dataset_id or dataset_name")
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            if dataset_id:
                res: RimeGetDatasetResponse = api.registry_service_get_dataset(
                    dataset_id=dataset_id
                )
            else:
                res = api.registry_service_get_dataset(dataset_name=dataset_name)
        return res.dataset.to_dict()

    def has_dataset(
        self, dataset_id: Optional[str] = None, dataset_name: Optional[str] = None
    ) -> bool:
        """Return a boolean on whether the dataset is present.

        Args:
            dataset_id: Optional[str] = None
                The ID of the dataset to check for.
            dataset_name: Optional[str] = None
                The name of the dataset to check for.

        Returns:
            bool:
                A boolean on whether the dataset is present.

        Raises:
            ValueError
                This error is generated any error other than HTTPStatus.NOT_FOUND
                is returned from the Registry service.
        """
        if dataset_id is not None and dataset_name is not None:
            raise ValueError(
                "Arguments dataset_id and dataset_name"
                " are mutually exclusive.\n"
                f"Received dataset_id={dataset_id}"
                f" and dataset_name={dataset_name}."
            )
        elif dataset_name is None and dataset_id is None:
            raise ValueError("Must provide either dataset_id or dataset_name")
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            try:
                if dataset_id:
                    api.registry_service_get_dataset(dataset_id=dataset_id)
                else:
                    api.registry_service_get_dataset(dataset_name=dataset_name)
            except ApiException as e:
                if e.status == HTTPStatus.NOT_FOUND:
                    return False
                else:
                    raise e
        return True

    def get_model(
        self, model_id: Optional[str] = None, model_name: Optional[str] = None
    ) -> Dict:
        """Return a model.

        Args:
            model_id: str
                The ID of the model to retrieve.
            model_name: str
                The name of the model to retrieve.

        Returns:
            Dict:
                A dictionary representing the model.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            if model_id is not None:
                if model_name is not None:
                    raise ValueError(
                        "Arguments model_id and model_name are mutually exclusive.\n"
                        f"Received model_id={model_id} and model_name={model_name}."
                    )
                res: RimeGetModelResponse = api.registry_service_get_model(
                    model_id_uuid=model_id
                )
            elif model_name is not None:
                res = api.registry_service_get_model(model_name=model_name)
            else:
                raise ValueError("Must provide either model_id or model_name")
            return res.model.model.to_dict()

    def get_predictions(self, model_id: str, dataset_id: str) -> Dict:
        """Get a prediction set.

        Args:
            model_id: str
                The ID of the model used to generate the predictions.
            dataset_id: str
                The ID of the dataset used to generate the predictions.

        Returns:
            Dict:
                A dictionary that contains the prediction set.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            res = api.registry_service_get_prediction_set(
                dataset_id=dataset_id, model_id_uuid=model_id
            )
            res = cast(RimeGetPredictionSetResponse, res)
            return res.prediction.to_dict()

    def delete_dataset(self, dataset_id: str) -> None:
        """Delete a dataset.

        Args:
            dataset_id: str
                The ID of the dataset to delete.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            api.registry_service_delete_dataset(dataset_id=dataset_id)

    def delete_model(self, model_id: str) -> None:
        """Delete a model.

        Args:
            model_id: str
                The ID of the model to delete.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            api.registry_service_delete_model(model_id_uuid=model_id)

    def delete_predictions(self, model_id: str, dataset_id: str) -> None:
        """Delete a prediction set.

        Args:
            model_id: str
                The ID of the model used to generate the predictions.
            dataset_id: str
                The ID of the dataset used to generate the predictions.

        Raises:
            ValueError
                This error is generated when the request to the Registry
                service fails.
        """
        api = swagger_client.RegistryServiceApi(self._api_client)
        with RESTErrorHandler():
            api.registry_service_delete_prediction_set(
                dataset_id=dataset_id, model_id_uuid=model_id
            )



"""CLI interface for the data format checker."""
from argparse import ArgumentParser
from pathlib import Path
from typing import Union

from rime_sdk.data_format_check.nlp_checker import NLP_TASKS, NlpDataFormatChecker
from rime_sdk.data_format_check.tabular_checker import (
    TABULAR_TASKS,
    TabularDataFormatChecker,
)


def pprint_exception(e: Exception) -> None:
    """Neatly prints out contents of exception."""
    args_joined = "".join([str(i) for i in e.args])
    print(f"\n---\n\nError:\n\n{args_joined}\n")


def main() -> None:
    """Parse CLI inputs and executes appropriate methods."""
    parser = ArgumentParser()

    # Data type
    group_types = parser.add_mutually_exclusive_group(required=True)
    group_types.add_argument(
        "-nlp", action="store_true", help="Whether this data is for NLP tasks"
    )
    group_types.add_argument(
        "-tabular", action="store_true", help="Whether this data is tabular"
    )

    # Shared args
    parser.add_argument(
        "--ref-path", type=Path, required=True, help="Path to reference data file."
    )
    parser.add_argument(
        "--eval-path", type=Path, required=True, help="Path to evaluation data file."
    )
    parser.add_argument(
        "--task",
        choices=list(TABULAR_TASKS) + list(NLP_TASKS),
        required=True,
        help="The desired ML task.",
    )

    # NLP args
    parser.add_argument(
        "--preds-ref-path",
        type=Path,
        required=False,
        help="(Optional) The path to the reference predictions, if they are stored"
        " in a separate file.",
    )
    parser.add_argument(
        "--preds-eval-path",
        type=Path,
        required=False,
        help="(Optional) The path to the evaluation predictions, if they are stored"
        " in a separate file.",
    )

    # Tabular args
    parser.add_argument(
        "--label-col-name",
        type=str,
        required=False,
        default=None,
        help="Name of column in inputs that contains labels.",
    )
    parser.add_argument(
        "--pred-col-name",
        type=str,
        required=False,
        default=None,
        help="Name of column in inputs that contains predictions.",
    )
    parser.add_argument(
        "--timestamp-col-name",
        type=str,
        required=False,
        default=None,
        help="Name of column in inputs that contains timestamps. "
        "Only applicable if using RIME Continuous Testing.",
    )

    argps = parser.parse_args()

    if argps.nlp:
        checker: Union[
            TabularDataFormatChecker, NlpDataFormatChecker
        ] = NlpDataFormatChecker()
        try:
            checker.check(
                argps.ref_path,
                argps.eval_path,
                argps.task,
                preds_ref_path=argps.preds_ref_path,
                preds_eval_path=argps.preds_eval_path,
            )
        except Exception as e:
            pprint_exception(e)
    elif argps.tabular:
        checker = TabularDataFormatChecker()
        try:
            checker.check(
                argps.ref_path,
                argps.eval_path,
                argps.task,
                label_col_name=argps.label_col_name,
                pred_col_name=argps.pred_col_name,
                timestamp_col_name=argps.timestamp_col_name,
            )
        except Exception as e:
            pprint_exception(e)



"""Parse protos / configs for the RIME file scanning service."""
from rime_sdk.swagger.swagger_client.models import (
    FilescanningHuggingfaceModelInfo,
    FilescanningModelFileInfo,
    FilescanningPytorchModelInfo,
)


def model_info_from_dict(model_info_dict: dict) -> FilescanningModelFileInfo:
    """Convert config to file scan proto."""
    scan_type = model_info_dict.get("scan_type")
    if scan_type is None:
        raise ValueError("The model_file_info must specify a scan type")
    scan_path = model_info_dict.get("scan_path")
    if scan_path is None:
        raise ValueError("The model_file_info must specify a scan path")
    typed_config = FilescanningModelFileInfo()
    if scan_type == "huggingface":
        hf_model_info = FilescanningHuggingfaceModelInfo(scan_path=scan_path)
        typed_config.huggingface_file = hf_model_info
    elif scan_type == "pytorch":
        pytorch_model_info = FilescanningPytorchModelInfo(scan_path=scan_path)
        typed_config.pytorch_file = pytorch_model_info
    else:
        raise ValueError(f"Invalid scan type {scan_type}")
    return typed_config


def convert_model_info_to_dict(model_file_info: FilescanningModelFileInfo) -> dict:
    """Convert file scan proto to config dict."""
    which_file = "pytorch_file" if model_file_info.pytorch_file else "huggingface_file"
    if which_file == "pytorch_file":
        return {
            "scan_type": "pytorch",
            "scan_path": model_file_info.pytorch_file.scan_path,
        }
    elif which_file == "huggingface_file":
        return {
            "scan_type": "huggingface",
            "scan_path": model_file_info.huggingface_file.scan_path,
        }
    else:
        raise ValueError(f"Unknown file info type: {which_file}")



"""Tabular-specific data format checking class and artifacts."""
from pathlib import Path
from typing import Any, Optional

import numpy as np
import pandas as pd

from rime_sdk.data_format_check.data_format_checker import DataFormatChecker

# Tabular Tasks
# TODO(RAT-1942): add Ranking
BINARY_CLASSIFICATION = "Binary Classification"
REGRESSION = "Regression"
MULTI_CLASS_CLASSIFICATION = "Multi-class Classification"
TABULAR_TASKS = {BINARY_CLASSIFICATION, REGRESSION, MULTI_CLASS_CLASSIFICATION}

# Accepted formats for the "timestamp" column
FORMAT_SECONDS = "%Y-%m-%d %H:%M:%S"
FORMAT_DAYS = "%Y-%m-%d"

WARNING_NO_PRED_COL = (
    "WARNING: No prediction column is provided. Although you can still run RIME "
    "without predictions, it will not be as powerful as if you run it WITH "
    "predictions.\n"
)
WARNING_NO_LABEL_COL = (
    "WARNING: No label column is provided. Although you can still run RIME without "
    "labels, it will not be as powerful as if you run it WITH labels.\n"
)
ERROR_UNKNOWN_TASK = "Unrecognized Tabular task: '{task}'. Task must be one of {tasks}"


class TabularDataFormatChecker(DataFormatChecker):
    """Checker for ML tasks involving Tabular data."""

    accepted_file_types = [".csv", ".parquet"]

    def check(
        self,
        ref_path: Path,
        eval_path: Path,
        task: str,
        label_col_name: Optional[str] = None,
        pred_col_name: Optional[str] = None,
        timestamp_col_name: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """Run all data checks."""

        self.check_file_format(ref_path)
        self.check_file_format(eval_path)

        for file_path in [ref_path, eval_path]:
            print(f"\nInspecting '{file_path}'")
            self.check_data_file(file_path, task, label_col_name, pred_col_name)
            print("Done!")

        print("")
        if pred_col_name is None:
            print(WARNING_NO_PRED_COL)
        if label_col_name is None:
            print(WARNING_NO_LABEL_COL)

        if timestamp_col_name is not None:
            print(
                f"Timestamp column provided: '{timestamp_col_name}'. Inspecting both "
                f"datasets for format of timestamps ('{FORMAT_SECONDS}' or "
                f"'{FORMAT_DAYS}')"
            )

        print("\n---\n")
        print(self.success_msg)

    def check_data_file(
        self,
        filename: Path,
        task: str,
        label_col_name: Optional[str],
        pred_col_name: Optional[str],
        timestamp_col_name: Optional[str] = None,
    ) -> None:
        """Perform multiple checks against the given file."""
        if not filename.exists():
            raise ValueError(f"File {filename} does not exist")

        if filename.suffix == ".csv":
            df = pd.read_csv(filename)
        elif filename.suffix == ".parquet":
            df = pd.read_parquet(filename)
        else:
            raise ValueError(
                f"Invalid file type '{filename.suffix}'. File must be one of "
                f"{self.accepted_file_types}"
            )

        if label_col_name is not None:
            if label_col_name not in df:
                raise ValueError(
                    f"Label column ({label_col_name}) not found in data "
                    f"({filename}). If a label column exists in one "
                    "dataset, it MUST exist in the other."
                )
            else:
                self.check_labels(df[label_col_name], task)

        if pred_col_name is not None:
            if pred_col_name not in df:
                raise ValueError(
                    f"Prediction column ({pred_col_name}) not found in data "
                    f"({filename}). If a prediction column exists in one dataset, "
                    f"it MUST exist in the other."
                )
            else:
                self.check_predictions(df[pred_col_name], task)

        if timestamp_col_name is not None:
            self.check_timestamps(df[timestamp_col_name], timestamp_col_name)

    def check_labels(self, ser: pd.Series, task: str) -> None:
        """Perform checks for the label data, based on the model type."""
        if ser.isnull().any():
            raise ValueError(
                "Found nulls in label series, there should not be any nulls."
            )
        if task == REGRESSION:
            if ser.dtype == object:
                raise ValueError("Labels for regression should be numeric.")
        elif task == BINARY_CLASSIFICATION:
            if not ((ser == 1) | (ser == 0)).all():
                raise ValueError(
                    "Labels for Binary Classification should be numeric, all 0s or 1s."
                )
        elif task == MULTI_CLASS_CLASSIFICATION:
            if not (ser == ser.astype(int)).all():
                raise ValueError(
                    "Labels for Multi-class Classification should be all integer "
                    "values."
                )
        else:
            raise ValueError(ERROR_UNKNOWN_TASK.format(task=task, tasks=TABULAR_TASKS))

    def check_predictions(self, ser: pd.Series, task: str) -> None:
        """Perform checks for the prediction data, based on the model type."""
        if ser.isnull().any():
            raise ValueError(
                "Found nulls in prediction series, there should not be any nulls."
            )
        if task == REGRESSION:
            if ser.dtype == object:
                raise ValueError("Predictions for regression should be numeric.")
        elif task == BINARY_CLASSIFICATION:
            if not ((ser <= 1) & (ser >= 0)).all():
                raise ValueError(
                    "Predictions for Binary Classification should be probabilities "
                    "between 0 and 1."
                )
        elif task == MULTI_CLASS_CLASSIFICATION:
            raise ValueError(
                "Prediction column for Multi-class Classification is not supported in"
                " usual way, please contact Robust Intelligence for instructions."
            )
        else:
            raise ValueError(ERROR_UNKNOWN_TASK.format(task=task, tasks=TABULAR_TASKS))

    def check_timestamps(
        self, timestamps: pd.Series, timestamp_col_name: Optional[str] = None
    ) -> None:
        """Validate format of timestamps."""
        try:
            timestamps = timestamps.astype(str)
            # Note: providing a subset of values (e.g. just year and month) is valid
            timestamps = pd.to_datetime(timestamps, format=FORMAT_SECONDS)
        except ValueError as e:
            # pd.to_datetime doesn't throw specific error type for this case,
            # must parse manually
            if "doesn't match format" in str(e):
                try:
                    timestamps = pd.to_datetime(timestamps, format=FORMAT_DAYS)
                except ValueError as new_error:
                    raise ValueError(
                        f"{timestamp_col_name} contains invalid formats."
                        f" Acceptable timestamp formats are {FORMAT_DAYS}"
                        f" and {FORMAT_SECONDS}.\n\n{new_error}"
                    )
            else:
                raise e

        if pd.isnull(timestamps).any():
            null_pos = np.where(pd.isnull(timestamps))[0]
            null_idxs = timestamps.index[null_pos].tolist()
            raise ValueError(
                f"{timestamp_col_name} must not contain nulls."
                f" Found null values at indexes: {null_idxs}"
            )

        if timestamps.nunique() < len(timestamps):
            print(f"WARNING: {timestamp_col_name} contains duplicate values!")



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from rime_sdk.swagger.swagger_client.api_client import ApiClient


class ModelTestingApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def model_testing_get_latest_logs(self, job_id, **kwargs):  # noqa: E501
        """GetLatestLogs  # noqa: E501

        Returns the logs of the latest pod to run the test job with the specified job ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_get_latest_logs(job_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str job_id: Uniquely specifies a Job. (required)
        :return: StreamResultOfRimeGetLatestLogsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.model_testing_get_latest_logs_with_http_info(job_id, **kwargs)  # noqa: E501
        else:
            (data) = self.model_testing_get_latest_logs_with_http_info(job_id, **kwargs)  # noqa: E501
            return data

    def model_testing_get_latest_logs_with_http_info(self, job_id, **kwargs):  # noqa: E501
        """GetLatestLogs  # noqa: E501

        Returns the logs of the latest pod to run the test job with the specified job ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_get_latest_logs_with_http_info(job_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str job_id: Uniquely specifies a Job. (required)
        :return: StreamResultOfRimeGetLatestLogsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['job_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method model_testing_get_latest_logs" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'job_id' is set
        if ('job_id' not in params or
                params['job_id'] is None):
            raise ValueError("Missing the required parameter `job_id` when calling `model_testing_get_latest_logs`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'job_id' in params:
            path_params['jobId'] = params['job_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Firewall-Api-Key', 'rime-api-key']  # noqa: E501

        return self.api_client.call_api(
            '/internal/logs/{jobId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='StreamResultOfRimeGetLatestLogsResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def model_testing_start_continuous_test(self, body, firewall_id_uuid, **kwargs):  # noqa: E501
        """StartContinuousTest  # noqa: E501

        Starts a Continuous Test and returns a Job object containing metadata for the Test Run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_start_continuous_test(body, firewall_id_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ContinuoustestsFirewallIdUuidBody body: (required)
        :param str firewall_id_uuid: Unique object ID. (required)
        :return: RimeStartContinuousTestResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.model_testing_start_continuous_test_with_http_info(body, firewall_id_uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.model_testing_start_continuous_test_with_http_info(body, firewall_id_uuid, **kwargs)  # noqa: E501
            return data

    def model_testing_start_continuous_test_with_http_info(self, body, firewall_id_uuid, **kwargs):  # noqa: E501
        """StartContinuousTest  # noqa: E501

        Starts a Continuous Test and returns a Job object containing metadata for the Test Run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_start_continuous_test_with_http_info(body, firewall_id_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ContinuoustestsFirewallIdUuidBody body: (required)
        :param str firewall_id_uuid: Unique object ID. (required)
        :return: RimeStartContinuousTestResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'firewall_id_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method model_testing_start_continuous_test" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `model_testing_start_continuous_test`")  # noqa: E501
        # verify the required parameter 'firewall_id_uuid' is set
        if ('firewall_id_uuid' not in params or
                params['firewall_id_uuid'] is None):
            raise ValueError("Missing the required parameter `firewall_id_uuid` when calling `model_testing_start_continuous_test`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'firewall_id_uuid' in params:
            path_params['firewallId.uuid'] = params['firewall_id_uuid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Firewall-Api-Key', 'rime-api-key']  # noqa: E501

        return self.api_client.call_api(
            '/v1/continuous-tests/{firewallId.uuid}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='RimeStartContinuousTestResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def model_testing_start_generative_stress_test(self, body, project_id_uuid, **kwargs):  # noqa: E501
        """StartGenerativeStressTest  # noqa: E501

        Starts a Generative Stress Test and returns a Job object containing metadata for the Test Run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_start_generative_stress_test(body, project_id_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GenerativestresstestsProjectIdUuidBody body: (required)
        :param str project_id_uuid: Unique object ID. (required)
        :return: RimeStartGenerativeStressTestResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.model_testing_start_generative_stress_test_with_http_info(body, project_id_uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.model_testing_start_generative_stress_test_with_http_info(body, project_id_uuid, **kwargs)  # noqa: E501
            return data

    def model_testing_start_generative_stress_test_with_http_info(self, body, project_id_uuid, **kwargs):  # noqa: E501
        """StartGenerativeStressTest  # noqa: E501

        Starts a Generative Stress Test and returns a Job object containing metadata for the Test Run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_start_generative_stress_test_with_http_info(body, project_id_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GenerativestresstestsProjectIdUuidBody body: (required)
        :param str project_id_uuid: Unique object ID. (required)
        :return: RimeStartGenerativeStressTestResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_id_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method model_testing_start_generative_stress_test" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `model_testing_start_generative_stress_test`")  # noqa: E501
        # verify the required parameter 'project_id_uuid' is set
        if ('project_id_uuid' not in params or
                params['project_id_uuid'] is None):
            raise ValueError("Missing the required parameter `project_id_uuid` when calling `model_testing_start_generative_stress_test`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id_uuid' in params:
            path_params['projectId.uuid'] = params['project_id_uuid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Firewall-Api-Key', 'rime-api-key']  # noqa: E501

        return self.api_client.call_api(
            '/internal/generative-stress-tests/{projectId.uuid}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='RimeStartGenerativeStressTestResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def model_testing_start_stress_test(self, body, project_id_uuid, **kwargs):  # noqa: E501
        """StartStressTest  # noqa: E501

        Starts a Stress Test and returns a Job object containing metadata for the Test Run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_start_stress_test(body, project_id_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param StresstestsProjectIdUuidBody body: (required)
        :param str project_id_uuid: Unique object ID. (required)
        :return: RimeStartStressTestResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.model_testing_start_stress_test_with_http_info(body, project_id_uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.model_testing_start_stress_test_with_http_info(body, project_id_uuid, **kwargs)  # noqa: E501
            return data

    def model_testing_start_stress_test_with_http_info(self, body, project_id_uuid, **kwargs):  # noqa: E501
        """StartStressTest  # noqa: E501

        Starts a Stress Test and returns a Job object containing metadata for the Test Run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.model_testing_start_stress_test_with_http_info(body, project_id_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param StresstestsProjectIdUuidBody body: (required)
        :param str project_id_uuid: Unique object ID. (required)
        :return: RimeStartStressTestResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_id_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method model_testing_start_stress_test" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `model_testing_start_stress_test`")  # noqa: E501
        # verify the required parameter 'project_id_uuid' is set
        if ('project_id_uuid' not in params or
                params['project_id_uuid'] is None):
            raise ValueError("Missing the required parameter `project_id_uuid` when calling `model_testing_start_stress_test`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id_uuid' in params:
            path_params['projectId.uuid'] = params['project_id_uuid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Firewall-Api-Key', 'rime-api-key']  # noqa: E501

        return self.api_client.call_api(
            '/v1/stress-tests/{projectId.uuid}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='RimeStartStressTestResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `rime-api-key` header.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RimeJobMetadata(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'job_id': 'str',
        'job_type': 'RimeJobType',
        'status': 'StatedbJobStatus',
        'start_time': 'datetime',
        'creation_time': 'datetime',
        'completion_time': 'datetime',
        'running_time_secs': 'float',
        'job_data': 'RimeJobData',
        'job_progress_str': 'str',
        'cancellation_requested': 'bool',
        'agent_id': 'RimeUUID',
        'archived_job_logs': 'StatedbArchivedJobLogs',
        'error_msg': 'str'
    }

    attribute_map = {
        'job_id': 'jobId',
        'job_type': 'jobType',
        'status': 'status',
        'start_time': 'startTime',
        'creation_time': 'creationTime',
        'completion_time': 'completionTime',
        'running_time_secs': 'runningTimeSecs',
        'job_data': 'jobData',
        'job_progress_str': 'jobProgressStr',
        'cancellation_requested': 'cancellationRequested',
        'agent_id': 'agentId',
        'archived_job_logs': 'archivedJobLogs',
        'error_msg': 'errorMsg'
    }

    def __init__(self, job_id=None, job_type=None, status=None, start_time=None, creation_time=None, completion_time=None, running_time_secs=None, job_data=None, job_progress_str=None, cancellation_requested=None, agent_id=None, archived_job_logs=None, error_msg=None):  # noqa: E501
        """RimeJobMetadata - a model defined in Swagger"""  # noqa: E501
        self._job_id = None
        self._job_type = None
        self._status = None
        self._start_time = None
        self._creation_time = None
        self._completion_time = None
        self._running_time_secs = None
        self._job_data = None
        self._job_progress_str = None
        self._cancellation_requested = None
        self._agent_id = None
        self._archived_job_logs = None
        self._error_msg = None
        self.discriminator = None
        if job_id is not None:
            self.job_id = job_id
        if job_type is not None:
            self.job_type = job_type
        if status is not None:
            self.status = status
        if start_time is not None:
            self.start_time = start_time
        if creation_time is not None:
            self.creation_time = creation_time
        if completion_time is not None:
            self.completion_time = completion_time
        if running_time_secs is not None:
            self.running_time_secs = running_time_secs
        if job_data is not None:
            self.job_data = job_data
        if job_progress_str is not None:
            self.job_progress_str = job_progress_str
        if cancellation_requested is not None:
            self.cancellation_requested = cancellation_requested
        if agent_id is not None:
            self.agent_id = agent_id
        if archived_job_logs is not None:
            self.archived_job_logs = archived_job_logs
        if error_msg is not None:
            self.error_msg = error_msg

    @property
    def job_id(self):
        """Gets the job_id of this RimeJobMetadata.  # noqa: E501

        The identifier within our job tracking system.  # noqa: E501

        :return: The job_id of this RimeJobMetadata.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this RimeJobMetadata.

        The identifier within our job tracking system.  # noqa: E501

        :param job_id: The job_id of this RimeJobMetadata.  # noqa: E501
        :type: str
        """

        self._job_id = job_id

    @property
    def job_type(self):
        """Gets the job_type of this RimeJobMetadata.  # noqa: E501


        :return: The job_type of this RimeJobMetadata.  # noqa: E501
        :rtype: RimeJobType
        """
        return self._job_type

    @job_type.setter
    def job_type(self, job_type):
        """Sets the job_type of this RimeJobMetadata.


        :param job_type: The job_type of this RimeJobMetadata.  # noqa: E501
        :type: RimeJobType
        """

        self._job_type = job_type

    @property
    def status(self):
        """Gets the status of this RimeJobMetadata.  # noqa: E501


        :return: The status of this RimeJobMetadata.  # noqa: E501
        :rtype: StatedbJobStatus
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this RimeJobMetadata.


        :param status: The status of this RimeJobMetadata.  # noqa: E501
        :type: StatedbJobStatus
        """

        self._status = status

    @property
    def start_time(self):
        """Gets the start_time of this RimeJobMetadata.  # noqa: E501

        The start time of the job (when the job transitions in state to RUNNING). Note, this may not be populated immediately when the job is created.  # noqa: E501

        :return: The start_time of this RimeJobMetadata.  # noqa: E501
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this RimeJobMetadata.

        The start time of the job (when the job transitions in state to RUNNING). Note, this may not be populated immediately when the job is created.  # noqa: E501

        :param start_time: The start_time of this RimeJobMetadata.  # noqa: E501
        :type: datetime
        """

        self._start_time = start_time

    @property
    def creation_time(self):
        """Gets the creation_time of this RimeJobMetadata.  # noqa: E501

        The time the job was created.  # noqa: E501

        :return: The creation_time of this RimeJobMetadata.  # noqa: E501
        :rtype: datetime
        """
        return self._creation_time

    @creation_time.setter
    def creation_time(self, creation_time):
        """Sets the creation_time of this RimeJobMetadata.

        The time the job was created.  # noqa: E501

        :param creation_time: The creation_time of this RimeJobMetadata.  # noqa: E501
        :type: datetime
        """

        self._creation_time = creation_time

    @property
    def completion_time(self):
        """Gets the completion_time of this RimeJobMetadata.  # noqa: E501

        The time the job entered a terminal state.  # noqa: E501

        :return: The completion_time of this RimeJobMetadata.  # noqa: E501
        :rtype: datetime
        """
        return self._completion_time

    @completion_time.setter
    def completion_time(self, completion_time):
        """Sets the completion_time of this RimeJobMetadata.

        The time the job entered a terminal state.  # noqa: E501

        :param completion_time: The completion_time of this RimeJobMetadata.  # noqa: E501
        :type: datetime
        """

        self._completion_time = completion_time

    @property
    def running_time_secs(self):
        """Gets the running_time_secs of this RimeJobMetadata.  # noqa: E501

        The total running time a job took to complete if the job is finished or the current running time if the job is still in progress (seconds).  # noqa: E501

        :return: The running_time_secs of this RimeJobMetadata.  # noqa: E501
        :rtype: float
        """
        return self._running_time_secs

    @running_time_secs.setter
    def running_time_secs(self, running_time_secs):
        """Sets the running_time_secs of this RimeJobMetadata.

        The total running time a job took to complete if the job is finished or the current running time if the job is still in progress (seconds).  # noqa: E501

        :param running_time_secs: The running_time_secs of this RimeJobMetadata.  # noqa: E501
        :type: float
        """

        self._running_time_secs = running_time_secs

    @property
    def job_data(self):
        """Gets the job_data of this RimeJobMetadata.  # noqa: E501


        :return: The job_data of this RimeJobMetadata.  # noqa: E501
        :rtype: RimeJobData
        """
        return self._job_data

    @job_data.setter
    def job_data(self, job_data):
        """Sets the job_data of this RimeJobMetadata.


        :param job_data: The job_data of this RimeJobMetadata.  # noqa: E501
        :type: RimeJobData
        """

        self._job_data = job_data

    @property
    def job_progress_str(self):
        """Gets the job_progress_str of this RimeJobMetadata.  # noqa: E501

        Pretty-printed, human-readable representation of job progress. This will only be populated for Read methods with the FULL job view. To get schema for progress for each type of job, see the field inside that job's job data message (e.g. StressTestJobProgress). Note: this is unstable, do not rely on parsing this.  # noqa: E501

        :return: The job_progress_str of this RimeJobMetadata.  # noqa: E501
        :rtype: str
        """
        return self._job_progress_str

    @job_progress_str.setter
    def job_progress_str(self, job_progress_str):
        """Sets the job_progress_str of this RimeJobMetadata.

        Pretty-printed, human-readable representation of job progress. This will only be populated for Read methods with the FULL job view. To get schema for progress for each type of job, see the field inside that job's job data message (e.g. StressTestJobProgress). Note: this is unstable, do not rely on parsing this.  # noqa: E501

        :param job_progress_str: The job_progress_str of this RimeJobMetadata.  # noqa: E501
        :type: str
        """

        self._job_progress_str = job_progress_str

    @property
    def cancellation_requested(self):
        """Gets the cancellation_requested of this RimeJobMetadata.  # noqa: E501

        Marked when the job has been requested to be cancelled by the user. This is declarative; once the user requests cancellation, the backend will conduct garbage collection on the job in the background and eventually update the status of the job to CANCELLED.  # noqa: E501

        :return: The cancellation_requested of this RimeJobMetadata.  # noqa: E501
        :rtype: bool
        """
        return self._cancellation_requested

    @cancellation_requested.setter
    def cancellation_requested(self, cancellation_requested):
        """Sets the cancellation_requested of this RimeJobMetadata.

        Marked when the job has been requested to be cancelled by the user. This is declarative; once the user requests cancellation, the backend will conduct garbage collection on the job in the background and eventually update the status of the job to CANCELLED.  # noqa: E501

        :param cancellation_requested: The cancellation_requested of this RimeJobMetadata.  # noqa: E501
        :type: bool
        """

        self._cancellation_requested = cancellation_requested

    @property
    def agent_id(self):
        """Gets the agent_id of this RimeJobMetadata.  # noqa: E501


        :return: The agent_id of this RimeJobMetadata.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._agent_id

    @agent_id.setter
    def agent_id(self, agent_id):
        """Sets the agent_id of this RimeJobMetadata.


        :param agent_id: The agent_id of this RimeJobMetadata.  # noqa: E501
        :type: RimeUUID
        """

        self._agent_id = agent_id

    @property
    def archived_job_logs(self):
        """Gets the archived_job_logs of this RimeJobMetadata.  # noqa: E501


        :return: The archived_job_logs of this RimeJobMetadata.  # noqa: E501
        :rtype: StatedbArchivedJobLogs
        """
        return self._archived_job_logs

    @archived_job_logs.setter
    def archived_job_logs(self, archived_job_logs):
        """Sets the archived_job_logs of this RimeJobMetadata.


        :param archived_job_logs: The archived_job_logs of this RimeJobMetadata.  # noqa: E501
        :type: StatedbArchivedJobLogs
        """

        self._archived_job_logs = archived_job_logs

    @property
    def error_msg(self):
        """Gets the error_msg of this RimeJobMetadata.  # noqa: E501

        User-facing error message for the job.  # noqa: E501

        :return: The error_msg of this RimeJobMetadata.  # noqa: E501
        :rtype: str
        """
        return self._error_msg

    @error_msg.setter
    def error_msg(self, error_msg):
        """Sets the error_msg of this RimeJobMetadata.

        User-facing error message for the job.  # noqa: E501

        :param error_msg: The error_msg of this RimeJobMetadata.  # noqa: E501
        :type: str
        """

        self._error_msg = error_msg

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RimeJobMetadata, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RimeJobMetadata):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
